@@ -34,7 +34,7 @@ QT_BEGIN_NAMESPACE
     operating system version (as opposed to the kernel version number or
     marketing version).
 
-    Presently, Android, Apple Platforms (iOS, macOS, tvOS, and watchOS),
+    Presently, Android, Apple Platforms (iOS, macOS, tvOS, watchOS, and visionOS),
     and Windows are supported.
 
     The \a majorVersion(), \a minorVersion(), and \a microVersion() functions
@@ -98,6 +98,7 @@ QT_BEGIN_NAMESPACE
     \value MacOS        The Apple macOS operating system.
     \value TvOS         The Apple tvOS operating system.
     \value WatchOS      The Apple watchOS operating system.
+    \value VisionOS     The Apple visionOS operating system.
     \value Windows      The Microsoft Windows operating system.
 
     \value Unknown      An unknown or unsupported operating system.
@@ -111,15 +112,11 @@ QT_BEGIN_NAMESPACE
 */
 
 /*!
+    \fn QOperatingSystemVersion::current()
     Returns a QOperatingSystemVersion indicating the current OS and its version number.
 
     \sa currentType()
 */
-QOperatingSystemVersion QOperatingSystemVersion::current()
-{
-    return QOperatingSystemVersionBase::current();
-}
-
 QOperatingSystemVersionBase QOperatingSystemVersionBase::current()
 {
     static const QOperatingSystemVersionBase v = current_impl();
@@ -203,13 +200,13 @@ QOperatingSystemVersionBase QOperatingSystemVersionBase::current_impl()
 }
 #endif
 
-static inline int compareVersionComponents(int lhs, int rhs)
+static inline int compareVersionComponents(int lhs, int rhs) noexcept
 {
     return lhs >= 0 && rhs >= 0 ? lhs - rhs : 0;
 }
 
 int QOperatingSystemVersionBase::compare(QOperatingSystemVersionBase v1,
-                                         QOperatingSystemVersionBase v2)
+                                         QOperatingSystemVersionBase v2) noexcept
 {
     if (v1.m_major == v2.m_major) {
         if (v1.m_minor == v2.m_minor) {
@@ -306,11 +303,6 @@ int QOperatingSystemVersionBase::compare(QOperatingSystemVersionBase v1,
 
     \sa type()
 */
-QString QOperatingSystemVersion::name() const
-{
-    return QOperatingSystemVersionBase::name();
-}
-
 QString QOperatingSystemVersionBase::name(QOperatingSystemVersionBase osversion)
 {
     switch (osversion.type()) {
@@ -334,6 +326,8 @@ QString QOperatingSystemVersionBase::name(QOperatingSystemVersionBase osversion)
         return QStringLiteral("tvOS");
     case QOperatingSystemVersionBase::WatchOS:
         return QStringLiteral("watchOS");
+    case QOperatingSystemVersionBase::VisionOS:
+        return QStringLiteral("visionOS");
     case QOperatingSystemVersionBase::Android:
         return QStringLiteral("Android");
     case QOperatingSystemVersionBase::Unknown:
@@ -479,6 +473,12 @@ const QOperatingSystemVersionBase QOperatingSystemVersion::Windows11_21H2;
  */
 const QOperatingSystemVersionBase QOperatingSystemVersion::Windows11_22H2;
 
+/*!
+    \variable QOperatingSystemVersion::Windows11_23H2
+    \brief a version corresponding to Windows 11 Version 23H2 (version 10.0.22631).
+    \since 6.6
+ */
+
 /*!
     \variable QOperatingSystemVersion::OSXMavericks
     \brief a version corresponding to OS X Mavericks (version 10.9).
@@ -561,8 +561,14 @@ const QOperatingSystemVersionBase QOperatingSystemVersion::MacOSVentura;
 /*!
     \variable QOperatingSystemVersion::MacOSSonoma
     \brief a version corresponding to macOS Sonoma (version 14).
+    \since 6.5
+*/
+
+/*!
+    \variable QOperatingSystemVersion::MacOSSequoia
+    \brief a version corresponding to macOS Sequoia (version 15).
+    \since 6.8
 */
-const QOperatingSystemVersionBase QOperatingSystemVersion::MacOSVentura;
 
 /*!
     \variable QOperatingSystemVersion::AndroidJellyBean
@@ -701,6 +707,12 @@ const QOperatingSystemVersionBase QOperatingSystemVersion::Android12L;
  */
 const QOperatingSystemVersionBase QOperatingSystemVersion::Android13;
 
+/*!
+    \variable QOperatingSystemVersion::Android14
+    \brief a version corresponding to Android 14 (version 14.0, API level 34).
+    \since 6.7
+ */
+
 #endif // !QT_BOOTSTRAPPED
 
 #ifndef QT_NO_DEBUG_STREAM
