@@ -19,6 +19,10 @@ using namespace QtCbor;
 
     \brief The QCborArray class is used to hold an array of CBOR elements.
 
+    \compares strong
+    \compareswith strong QCborValueConstRef
+    \endcompareswith
+
     This class can be used to hold one sequential container in CBOR (an array).
     CBOR is the Concise Binary Object Representation, a very compact form of
     binary data encoding that is a superset of JSON. It was created by the IETF
@@ -31,8 +35,8 @@ using namespace QtCbor;
     from those two, though there may be loss of information in some
     conversions.
 
-    \sa QCborValue, QCborMap, QJsonArray, QList, {Cbordump Example},
-        {Convert Example}, {JSON Save Game Example}
+    \sa QCborValue, QCborMap, QJsonArray, QList, {Parsing and displaying CBOR data},
+        {Serialization Converter}, {Saving and Loading a Game}
  */
 
 /*!
@@ -420,7 +424,7 @@ void QCborArray::removeAt(qsizetype i)
 bool QCborArray::contains(const QCborValue &value) const
 {
     for (qsizetype i = 0; i < size(); ++i) {
-        int cmp = d->compareElement(i, value);
+        int cmp = d->compareElement(i, value, Comparison::ForEquality);
         if (cmp == 0)
             return true;
     }
@@ -442,9 +446,9 @@ bool QCborArray::contains(const QCborValue &value) const
  */
 
 /*!
-    \fn bool QCborArray::operator==(const QCborArray &other) const
+    \fn bool QCborArray::operator==(const QCborArray &lhs, const QCborArray &rhs)
 
-    Compares this array and \a other, comparing each element in sequence, and
+    Compares \a lhs and \a rhs arrays, comparing each element in sequence, and
     returns true if both arrays contains the same elements, false otherwise.
 
     For more information on CBOR equality in Qt, see, QCborValue::compare().
@@ -454,9 +458,9 @@ bool QCborArray::contains(const QCborValue &value) const
  */
 
 /*!
-    \fn bool QCborArray::operator!=(const QCborArray &other) const
+    \fn bool QCborArray::operator!=(const QCborArray &lhs, const QCborArray &rhs)
 
-    Compares this array and \a other, comparing each element in sequence, and
+    Compares \a lhs and \a rhs arrays, comparing each element in sequence, and
     returns true if the two arrays' contents are different, false otherwise.
 
     For more information on CBOR equality in Qt, see, QCborValue::compare().
@@ -466,18 +470,57 @@ bool QCborArray::contains(const QCborValue &value) const
  */
 
 /*!
-    \fn bool QCborArray::operator<(const QCborArray &other) const
+    \fn bool QCborArray::operator<(const QCborArray &lhs, const QCborArray &rhs)
 
-    Compares this array and \a other, comparing each element in sequence, and
-    returns true if this array should be sorted before \a other, false
+    Compares \a lhs and \a rhs arrays, comparing each element in sequence, and
+    returns true if \a lhs array should be sorted before \a rhs, false
     otherwise.
 
     For more information on CBOR sorting order, see QCborValue::compare().
 
     \sa compare(), QCborValue::operator==(), QCborMap::operator==(),
-        operator==(), operator!=()
+        operator==(), operator!=(), operator<=()
  */
 
+/*!
+    \fn bool QCborArray::operator<=(const QCborArray &lhs, const QCborArray &rhs)
+
+    Compares \a lhs and \a rhs arrays, comparing each element in sequence, and
+    returns true if \a lhs array should be sorted before \a rhs, or if both
+    arrays contains the same elements, false otherwise.
+
+    For more information on CBOR sorting order, see QCborValue::compare().
+
+    \sa compare(), QCborValue::operator==(), QCborMap::operator==(),
+        operator==(), operator!=(), operator<()
+*/
+
+/*!
+    \fn bool QCborArray::operator>(const QCborArray &lhs, const QCborArray &rhs)
+
+    Compares \a lhs and \a rhs arrays, comparing each element in sequence, and
+    returns true if \a lhs array should be sorted after \a rhs, false
+    otherwise.
+
+    For more information on CBOR sorting order, see QCborValue::compare().
+
+    \sa compare(), QCborValue::operator==(), QCborMap::operator==(),
+        operator==(), operator!=(), operator>=()
+*/
+
+/*!
+    \fn bool QCborArray::operator>=(const QCborArray &lhs, const QCborArray &rhs)
+
+    Compares \a lhs and \a rhs arrays, comparing each element in sequence, and
+    returns true if \a lhs array should be sorted after \a rhs, or if both
+    arrays contains the same elements, false otherwise.
+
+    For more information on CBOR sorting order, see QCborValue::compare().
+
+    \sa compare(), QCborValue::operator==(), QCborMap::operator==(),
+        operator==(), operator!=(), operator>()
+*/
+
 /*!
     \typedef QCborArray::iterator
 
@@ -679,6 +722,10 @@ void QCborArray::detach(qsizetype reserved)
 
     \brief The QCborArray::Iterator class provides an STL-style non-const iterator for QCborArray.
 
+    \compares strong
+    \compareswith strong QCborArray::ConstIterator
+    \endcompareswith
+
     QCborArray::Iterator allows you to iterate over a QCborArray and to modify
     the array item associated with the iterator. If you want to iterate over a
     const QCborArray, use QCborArray::ConstIterator instead. It is generally a
@@ -793,63 +840,63 @@ void QCborArray::detach(qsizetype reserved)
 */
 
 /*!
-    \fn bool QCborArray::Iterator::operator==(const Iterator &other) const
-    \fn bool QCborArray::Iterator::operator==(const ConstIterator &other) const
+    \fn bool QCborArray::Iterator::operator==(const Iterator &lhs, const Iterator &rhs)
+    \fn bool QCborArray::Iterator::operator==(const Iterator &lhs, const ConstIterator &rhs)
 
-    Returns \c true if \a other points to the same entry in the array as this
+    Returns \c true if \a lhs points to the same entry in the array as \a rhs
     iterator; otherwise returns \c false.
 
     \sa operator!=()
 */
 
 /*!
-    \fn bool QCborArray::Iterator::operator!=(const Iterator &other) const
-    \fn bool QCborArray::Iterator::operator!=(const ConstIterator &other) const
+    \fn bool QCborArray::Iterator::operator!=(const Iterator &lhs, const Iterator &rhs)
+    \fn bool QCborArray::Iterator::operator!=(const Iterator &lhs, const ConstIterator &rhs)
 
-    Returns \c true if \a other points to a different entry in the array than
-    this iterator; otherwise returns \c false.
+    Returns \c true if \a lhs points to a different entry in the array than
+    \a rhs iterator; otherwise returns \c false.
 
     \sa operator==()
 */
 
 /*!
-    \fn bool QCborArray::Iterator::operator<(const Iterator& other) const
-    \fn bool QCborArray::Iterator::operator<(const ConstIterator& other) const
+    \fn bool QCborArray::Iterator::operator<(const Iterator &lhs, const Iterator &rhs)
+    \fn bool QCborArray::Iterator::operator<(const Iterator &lhs, const ConstIterator &rhs)
 
-    Returns \c true if the entry in the array pointed to by this iterator
-    occurs before the entry pointed to by the \a other iterator.
+    Returns \c true if the entry in the array pointed to by \a lhs iterator
+    occurs before the entry pointed to by the \a rhs iterator.
 */
 
 /*!
-    \fn bool QCborArray::Iterator::operator<=(const Iterator& other) const
-    \fn bool QCborArray::Iterator::operator<=(const ConstIterator& other) const
+    \fn bool QCborArray::Iterator::operator<=(const Iterator &lhs, const Iterator &rhs)
+    \fn bool QCborArray::Iterator::operator<=(const Iterator &lhs, const ConstIterator &rhs)
 
-    Returns \c true if the entry in the array pointed to by this iterator
-    occurs before or is the same entry as is pointed to by the \a other
+    Returns \c true if the entry in the array pointed to by \a lhs iterator
+    occurs before or is the same entry as is pointed to by the \a rhs
     iterator.
 */
 
 /*!
-    \fn bool QCborArray::Iterator::operator>(const Iterator& other) const
-    \fn bool QCborArray::Iterator::operator>(const ConstIterator& other) const
+    \fn bool QCborArray::Iterator::operator>(const Iterator &lhs, const Iterator &rhs)
+    \fn bool QCborArray::Iterator::operator>(const Iterator &lhs, const ConstIterator &rhs)
 
-    Returns \c true if the entry in the array pointed to by this iterator
-    occurs after the entry pointed to by the \a other iterator.
+    Returns \c true if the entry in the array pointed to by \a lhs iterator
+    occurs after the entry pointed to by the \a rhs iterator.
  */
 
 /*!
-    \fn bool QCborArray::Iterator::operator>=(const Iterator& other) const
-    \fn bool QCborArray::Iterator::operator>=(const ConstIterator& other) const
+    \fn bool QCborArray::Iterator::operator>=(const Iterator &lhs, const Iterator &rhs)
+    \fn bool QCborArray::Iterator::operator>=(const Iterator &lhs, const ConstIterator &rhs)
 
-    Returns \c true if the entry in the array pointed to by this iterator
-    occurs after or is the same entry as is pointed to by the \a other
+    Returns \c true if the entry in the array pointed to by \a lhs iterator
+    occurs after or is the same entry as is pointed to by the \a rhs
     iterator.
 */
 
 /*!
     \fn QCborArray::Iterator &QCborArray::Iterator::operator++()
 
-    The prefix ++ operator, \c{++it}, advances the iterator to the next item in
+    The prefix \c{++} operator, \c{++it}, advances the iterator to the next item in
     the array and returns this iterator.
 
     Calling this function on QCborArray::end() leads to undefined results.
@@ -861,14 +908,14 @@ void QCborArray::detach(qsizetype reserved)
     \fn QCborArray::Iterator QCborArray::Iterator::operator++(int)
     \overload
 
-    The postfix ++ operator, \c{it++}, advances the iterator to the next item
+    The postfix \c{++} operator, \c{it++}, advances the iterator to the next item
     in the array and returns an iterator to the previously current item.
 */
 
 /*!
     \fn QCborArray::Iterator &QCborArray::Iterator::operator--()
 
-    The prefix -- operator, \c{--it}, makes the preceding item current and
+    The prefix \c{--} operator, \c{--it}, makes the preceding item current and
     returns this iterator.
 
     Calling this function on QCborArray::begin() leads to undefined results.
@@ -880,7 +927,7 @@ void QCborArray::detach(qsizetype reserved)
     \fn QCborArray::Iterator QCborArray::Iterator::operator--(int)
     \overload
 
-    The postfix -- operator, \c{it--}, makes the preceding item current and
+    The postfix \c{--} operator, \c{it--}, makes the preceding item current and
     returns an iterator to the previously current item.
 */
 
@@ -934,6 +981,10 @@ void QCborArray::detach(qsizetype reserved)
 
     \brief The QCborArray::ConstIterator class provides an STL-style const iterator for QCborArray.
 
+    \compares strong
+    \compareswith strong QCborArray::Iterator
+    \endcompareswith
+
     QCborArray::ConstIterator allows you to iterate over a QCborArray. If you
     want to modify the QCborArray as you iterate over it, use
     QCborArray::Iterator instead. It is generally good practice to use
@@ -1033,63 +1084,57 @@ void QCborArray::detach(qsizetype reserved)
 */
 
 /*!
-    \fn bool QCborArray::ConstIterator::operator==(const Iterator &other) const
-    \fn bool QCborArray::ConstIterator::operator==(const ConstIterator &other) const
+    \fn bool QCborArray::ConstIterator::operator==(const ConstIterator &lhs, const ConstIterator &rhs)
 
-    Returns \c true if \a other points to the same entry in the array as this
+    Returns \c true if \a lhs points to the same entry in the array as \a rhs
     iterator; otherwise returns \c false.
 
     \sa operator!=()
 */
 
 /*!
-    \fn bool QCborArray::ConstIterator::operator!=(const Iterator &o) const
-    \fn bool QCborArray::ConstIterator::operator!=(const ConstIterator &o) const
+    \fn bool QCborArray::ConstIterator::operator!=(const ConstIterator &lhs, const ConstIterator &rhs)
 
-    Returns \c true if \a o points to a different entry in the array than
-    this iterator; otherwise returns \c false.
+    Returns \c true if \a lhs points to a different entry in the array than
+    \a rhs iterator; otherwise returns \c false.
 
     \sa operator==()
 */
 
 /*!
-    \fn bool QCborArray::ConstIterator::operator<(const Iterator &other) const
-    \fn bool QCborArray::ConstIterator::operator<(const ConstIterator &other) const
+    \fn bool QCborArray::ConstIterator::operator<(const ConstIterator &lhs, const ConstIterator &rhs)
 
-    Returns \c true if the entry in the array pointed to by this iterator
-    occurs before the entry pointed to by the \a other iterator.
+    Returns \c true if the entry in the array pointed to by \a lhs iterator
+    occurs before the entry pointed to by the \a rhs iterator.
 */
 
 /*!
-    \fn bool QCborArray::ConstIterator::operator<=(const Iterator &other) const
-    \fn bool QCborArray::ConstIterator::operator<=(const ConstIterator &other) const
+    \fn bool QCborArray::ConstIterator::operator<=(const ConstIterator &lhs, const ConstIterator &rhs)
 
-    Returns \c true if the entry in the array pointed to by this iterator
-    occurs before or is the same entry as is pointed to by the \a other
+    Returns \c true if the entry in the array pointed to by \a lhs iterator
+    occurs before or is the same entry as is pointed to by the \a rhs
     iterator.
 */
 
 /*!
-    \fn bool QCborArray::ConstIterator::operator>(const Iterator &other) const
-    \fn bool QCborArray::ConstIterator::operator>(const ConstIterator &other) const
+    \fn bool QCborArray::ConstIterator::operator>(const ConstIterator &lhs, const ConstIterator &rhs)
 
-    Returns \c true if the entry in the array pointed to by this iterator
-    occurs after the entry pointed to by the \a other iterator.
+    Returns \c true if the entry in the array pointed to by \a lhs iterator
+    occurs after the entry pointed to by the \a rhs iterator.
 */
 
 /*!
-    \fn bool QCborArray::ConstIterator::operator>=(const Iterator &other) const
-    \fn bool QCborArray::ConstIterator::operator>=(const ConstIterator &other) const
+    \fn bool QCborArray::ConstIterator::operator>=(const ConstIterator &lhs, const ConstIterator &rhs)
 
-    Returns \c true if the entry in the array pointed to by this iterator
-    occurs after or is the same entry as is pointed to by the \a other
+    Returns \c true if the entry in the array pointed to by \a lhs iterator
+    occurs after or is the same entry as is pointed to by the \a rhs
     iterator.
 */
 
 /*!
     \fn QCborArray::ConstIterator &QCborArray::ConstIterator::operator++()
 
-    The prefix ++ operator, \c{++it}, advances the iterator to the next item in
+    The prefix \c{++} operator, \c{++it}, advances the iterator to the next item in
     the array and returns this iterator.
 
     Calling this function on QCborArray::end() leads to undefined results.
@@ -1101,14 +1146,14 @@ void QCborArray::detach(qsizetype reserved)
     \fn QCborArray::ConstIterator QCborArray::ConstIterator::operator++(int)
     \overload
 
-    The postfix ++ operator, \c{it++}, advances the iterator to the next item
+    The postfix \c{++} operator, \c{it++}, advances the iterator to the next item
     in the array and returns an iterator to the previously current item.
 */
 
 /*!
     \fn QCborArray::ConstIterator &QCborArray::ConstIterator::operator--()
 
-    The prefix -- operator, \c{--it}, makes the preceding item current and
+    The prefix \c{--} operator, \c{--it}, makes the preceding item current and
     returns this iterator.
 
     Calling this function on QCborArray::begin() leads to undefined results.
@@ -1120,7 +1165,7 @@ void QCborArray::detach(qsizetype reserved)
     \fn QCborArray::ConstIterator QCborArray::ConstIterator::operator--(int)
     \overload
 
-    The postfix -- operator, \c{it--}, makes the preceding item current and
+    The postfix \c{--} operator, \c{it--}, makes the preceding item current and
     returns an iterator to the previously current item.
 */
 
