@@ -1,50 +1,17 @@
-/****************************************************************************
-**
-** Copyright (C) 2021 The Qt Company Ltd.
-** Copyright (C) 2016 Intel Corporation.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtCore module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2021 The Qt Company Ltd.
+// Copyright (C) 2016 Intel Corporation.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qlocale_tools_p.h"
 #include "qdoublescanprint_p.h"
 #include "qlocale_p.h"
 #include "qstring.h"
 
+#include <private/qtools_p.h>
 #include <private/qnumeric_p.h>
 
+#include <cstdio>
+
 #include <ctype.h>
 #include <errno.h>
 #include <float.h>
@@ -73,6 +40,8 @@
 
 QT_BEGIN_NAMESPACE
 
+using namespace QtMiscUtils;
+
 QT_CLOCALE_HOLDER
 
 void qt_doubleToAscii(double d, QLocaleData::DoubleForm form, int precision,
@@ -227,11 +196,10 @@ void qt_doubleToAscii(double d, QLocaleData::DoubleForm form, int precision,
             // which case the missing digits are zeroes. In the 'e' case decptInTarget is always 1,
             // as variants of snprintf always generate numbers with one digit before the '.' then.
             // This is why the final decimal point is offset by 1, relative to the number after 'e'.
-            bool ok;
-            const char *endptr;
-            decpt = qstrntoll(target.data() + eSign + 1, length - eSign - 1, &endptr, 10, &ok) + 1;
-            Q_ASSERT(ok);
-            Q_ASSERT(endptr - target.data() <= length);
+            auto r = qstrntoll(target.data() + eSign + 1, length - eSign - 1, 10);
+            decpt = r.result + 1;
+            Q_ASSERT(r.ok());
+            Q_ASSERT(r.used + eSign + 1 <= length);
         } else {
             // No 'e' found, so it's the 'f' form. Variants of snprintf generate numbers with
             // potentially multiple digits before the '.', but without decimal exponent then. So we
@@ -284,48 +252,50 @@ void qt_doubleToAscii(double d, QLocaleData::DoubleForm form, int precision,
         --length;
 }
 
-double qt_asciiToDouble(const char *num, qsizetype numLen, bool &ok, int &processed,
-                        StrayCharacterMode strayCharMode)
+QSimpleParsedNumber<double> qt_asciiToDouble(const char *num, qsizetype numLen,
+                                             StrayCharacterMode strayCharMode)
 {
-    auto string_equals = [](const char *needle, const char *haystack, qsizetype haystackLen) {
-        qsizetype needleLen = strlen(needle);
-        return needleLen == haystackLen && memcmp(needle, haystack, haystackLen) == 0;
-    };
-
-    if (numLen <= 0) {
-        ok = false;
-        processed = 0;
-        return 0.0;
-    }
-
-    ok = true;
+    if (numLen <= 0)
+        return {};
 
     // We have to catch NaN before because we need NaN as marker for "garbage" in the
     // libdouble-conversion case and, in contrast to libdouble-conversion or sscanf, we don't allow
     // "-nan" or "+nan"
-    if (string_equals("nan", num, numLen)) {
-        processed = 3;
-        return qt_qnan();
-    } else if (string_equals("+nan", num, numLen) || string_equals("-nan", num, numLen)) {
-        processed = 0;
-        ok = false;
-        return 0.0;
-    }
+    if (char c = *num; numLen >= 3
+            && (c == '-' || c == '+' || c == 'I' || c == 'i' || c == 'N' || c == 'n')) {
+        bool negative = (c == '-');
+        bool hasSign = negative || (c == '+');
+        qptrdiff offset = 0;
+        if (hasSign) {
+            offset = 1;
+            c = num[offset];
+        }
 
-    // Infinity values are implementation defined in the sscanf case. In the libdouble-conversion
-    // case we need infinity as overflow marker.
-    if (string_equals("+inf", num, numLen)) {
-        processed = 4;
-        return qt_inf();
-    } else if (string_equals("inf", num, numLen)) {
-        processed = 3;
-        return qt_inf();
-    } else if (string_equals("-inf", num, numLen)) {
-        processed = 4;
-        return -qt_inf();
+        if (c > '9') {
+            auto lowered = [](char c) {
+                // this will mangle non-letters, but none can become a letter
+                return c | 0x20;
+            };
+
+            // Found a non-digit, so this MUST be either "inf", "+inf", "-inf"
+            // or "nan". Anything else is an invalid parse and we don't need to
+            // feed it to the converter below.
+            if (numLen != offset + 3)
+                return {};
+
+            c = lowered(c);
+            char c2 = lowered(num[offset + 1]);
+            char c3 = lowered(num[offset + 2]);
+            if (c == 'i' && c2 == 'n' && c3 == 'f')
+                return { negative ? -qt_inf() : qt_inf(), offset + 3 };
+            else if (c == 'n' && c2 == 'a' && c3 == 'n' && !hasSign)
+                return { qt_qnan(), 3 };
+            return {};
+        }
     }
 
     double d = 0.0;
+    int processed;
 #if !defined(QT_NO_DOUBLECONVERSION) && !defined(QT_BOOTSTRAPPED)
     int conv_flags = double_conversion::StringToDoubleConverter::NO_FLAGS;
     if (strayCharMode == TrailingJunkAllowed) {
@@ -337,22 +307,18 @@ double qt_asciiToDouble(const char *num, qsizetype numLen, bool &ok, int &proces
     double_conversion::StringToDoubleConverter conv(conv_flags, 0.0, qt_qnan(), nullptr, nullptr);
     if (int(numLen) != numLen) {
         // a number over 2 GB in length is silly, just assume it isn't valid
-        ok = false;
-        processed = 0;
-        return 0.0;
+        return {};
     } else {
-        d = conv.StringToDouble(num, numLen, &processed);
+        d = conv.StringToDouble(num, int(numLen), &processed);
     }
 
-    if (!qIsFinite(d)) {
-        ok = false;
-        if (qIsNaN(d)) {
+    if (!qt_is_finite(d)) {
+        if (qt_is_nan(d)) {
             // Garbage found. We don't accept it and return 0.
-            processed = 0;
-            return 0.0;
+            return {};
         } else {
             // Overflow. That's not OK, but we still return infinity.
-            return d;
+            return { d, -processed };
         }
     }
 #else
@@ -360,32 +326,29 @@ double qt_asciiToDouble(const char *num, qsizetype numLen, bool &ok, int &proces
     constexpr auto maxDigitsForULongLong = 1 + std::numeric_limits<unsigned long long>::digits10;
     // need to ensure that we don't read more than numLen of input:
     char fmt[1 + maxDigitsForULongLong + 4 + 1];
-    qsnprintf(fmt, sizeof fmt, "%s%llu%s", "%", static_cast<unsigned long long>(numLen), "lf%n");
+    std::snprintf(fmt, sizeof fmt, "%s%llu%s",
+                  "%", static_cast<unsigned long long>(numLen), "lf%n");
 
     if (qDoubleSscanf(num, QT_CLOCALE, fmt, &d, &processed) < 1)
         processed = 0;
 
-    if ((strayCharMode == TrailingJunkProhibited && processed != numLen) || qIsNaN(d)) {
+    if ((strayCharMode == TrailingJunkProhibited && processed != numLen) || qt_is_nan(d)) {
         // Implementation defined nan symbol or garbage found. We don't accept it.
-        processed = 0;
-        ok = false;
-        return 0.0;
+        return {};
     }
 
-    if (!qIsFinite(d)) {
+    if (!qt_is_finite(d)) {
         // Overflow. Check for implementation-defined infinity symbols and reject them.
         // We assume that any infinity symbol has to contain a character that cannot be part of a
         // "normal" number (that is 0-9, ., -, +, e).
-        ok = false;
         for (int i = 0; i < processed; ++i) {
             char c = num[i];
             if ((c < '0' || c > '9') && c != '.' && c != '-' && c != '+' && c != 'e' && c != 'E') {
                 // Garbage found
-                processed = 0;
-                return 0.0;
+                return {};
             }
         }
-        return d;
+        return { d, -processed };
     }
 #endif // !defined(QT_NO_DOUBLECONVERSION) && !defined(QT_BOOTSTRAPPED)
 
@@ -397,40 +360,51 @@ double qt_asciiToDouble(const char *num, qsizetype numLen, bool &ok, int &proces
         for (int i = 0; i < processed; ++i) {
             if (num[i] >= '1' && num[i] <= '9') {
                 // if a digit before any 'e' is not 0, then a non-zero number was intended.
-                ok = false;
-                return 0.0;
+                return {d, -processed};
             } else if (num[i] == 'e' || num[i] == 'E') {
                 break;
             }
         }
     }
-    return d;
+    return { d, processed };
 }
 
-/* Detect base if 0 and, if base is hex, skip over 0x prefix */
+/* Detect base if 0 and, if base is hex or bin, skip over 0x/0b prefixes */
 static auto scanPrefix(const char *p, const char *stop, int base)
 {
-    if (p < stop && *p >= '0' && *p <= '9') {
-        if (*p == '0') {
-            const char *x = p + 1;
-            if (x < stop && (*x == 'x' || *x == 'X')) {
-                if (base == 0)
-                    base = 16;
-                if (base == 16)
-                    p += 2;
-            } else if (base == 0) {
-                base = 8;
-            }
-        } else if (base == 0) {
-            base = 10;
-        }
-        Q_ASSERT(base);
-    }
     struct R
     {
         const char *next;
         int base;
     };
+    if (p < stop && isAsciiDigit(*p)) {
+        if (*p == '0') {
+            const char *x_or_b = p + 1;
+            if (x_or_b < stop) {
+                switch (*x_or_b) {
+                case 'b':
+                case 'B':
+                    if (base == 0)
+                        base = 2;
+                    if (base == 2)
+                        p += 2;
+                    return R{p, base};
+                case 'x':
+                case 'X':
+                    if (base == 0)
+                        base = 16;
+                    if (base == 16)
+                        p += 2;
+                    return R{p, base};
+                }
+            }
+            if (base == 0)
+                base = 8;
+        } else if (base == 0) {
+            base = 10;
+        }
+        Q_ASSERT(base);
+    }
     return R{p, base};
 }
 
@@ -447,36 +421,25 @@ static bool isDigitForBase(char d, int base)
     return false;
 }
 
-unsigned long long
-qstrntoull(const char *begin, qsizetype size, const char **endptr, int base, bool *ok)
+QSimpleParsedNumber<qulonglong> qstrntoull(const char *begin, qsizetype size, int base)
 {
     const char *p = begin, *const stop = begin + size;
     while (p < stop && ascii_isspace(*p))
         ++p;
     unsigned long long result = 0;
-    if (p >= stop || *p == '-') {
-        *ok = false;
-        if (endptr)
-            *endptr = begin;
-        return result;
-    }
+    if (p >= stop || *p == '-')
+        return { };
     const auto prefix = scanPrefix(*p == '+' ? p + 1 : p, stop, base);
-    if (!prefix.base || prefix.next >= stop) {
-        if (endptr)
-            *endptr = begin;
-        *ok = false;
-        return 0;
-    }
+    if (!prefix.base || prefix.next >= stop)
+        return { };
 
     const auto res = std::from_chars(prefix.next, stop, result, prefix.base);
-    *ok = res.ec == std::errc{};
-    if (endptr)
-        *endptr = res.ptr == prefix.next ? begin : res.ptr;
-    return result;
+    if (res.ec != std::errc{})
+        return { };
+    return { result, res.ptr == prefix.next ? 0 : res.ptr - begin };
 }
 
-long long
-qstrntoll(const char *begin, qsizetype size, const char **endptr, int base, bool *ok)
+QSimpleParsedNumber<qlonglong> qstrntoll(const char *begin, qsizetype size, int base)
 {
     const char *p = begin, *const stop = begin + size;
     while (p < stop && ascii_isspace(*p))
@@ -491,30 +454,22 @@ qstrntoll(const char *begin, qsizetype size, const char **endptr, int base, bool
     const auto prefix = scanPrefix(p, stop, base);
     // Must check for digit, as from_chars() will accept a sign, which would be
     // a second sign, that we should reject.
-    if (!prefix.base || prefix.next >= stop || !isDigitForBase(*prefix.next, prefix.base)) {
-        if (endptr)
-            *endptr = begin;
-        *ok = false;
-        return 0;
-    }
+    if (!prefix.base || prefix.next >= stop || !isDigitForBase(*prefix.next, prefix.base))
+        return { };
 
     long long result = 0;
     auto res = std::from_chars(prefix.next, stop, result, prefix.base);
-    *ok = res.ec == std::errc{};
     if (negate && res.ec == std::errc::result_out_of_range) {
         // Maybe LLONG_MIN:
         unsigned long long check = 0;
         res = std::from_chars(prefix.next, stop, check, prefix.base);
-        if (res.ec == std::errc{} && check + std::numeric_limits<long long>::min() == 0) {
-            *ok = true;
-            if (endptr)
-                *endptr = res.ptr;
-            return std::numeric_limits<long long>::min();
-        }
+        if (res.ec == std::errc{} && check + std::numeric_limits<long long>::min() == 0)
+            return { std::numeric_limits<long long>::min(), res.ptr - begin };
+        return { };
     }
-    if (endptr)
-        *endptr = res.ptr == prefix.next ? begin : res.ptr;
-    return negate && *ok ? -result : result;
+    if (res.ec != std::errc{})
+        return { };
+    return { negate ? -result : result, res.ptr - begin };
 }
 
 template <typename Char>
@@ -594,28 +549,39 @@ QString qulltoa(qulonglong number, int base, const QStringView zero)
             number /= base;
         }
     } else { // zero should always be either a non-surrogate or a surrogate pair:
-        Q_UNREACHABLE();
-        return QString();
+        Q_UNREACHABLE_RETURN(QString());
     }
 
     return QString(reinterpret_cast<QChar *>(p), end - p);
 }
 
+char *qulltoa2(char *p, qulonglong n, int base)
+{
+#if defined(QT_CHECK_RANGE)
+    if (base < 2 || base > 36) {
+        qWarning("QByteArray::setNum: Invalid base %d", base);
+        base = 10;
+    }
+#endif
+    qulltoString_helper(n, base, p);
+    return p;
+}
+
 /*!
   \internal
 
-  Converts the initial portion of the string pointed to by \a s00 to a double, using the 'C' locale.
+  Converts the initial portion of the string pointed to by \a s00 to a double,
+  using the 'C' locale. The function sets the pointer pointed to by \a se to
+  point to the character past the last character converted.
  */
 double qstrntod(const char *s00, qsizetype len, const char **se, bool *ok)
 {
-    int processed = 0;
-    bool nonNullOk = false;
-    double d = qt_asciiToDouble(s00, len, nonNullOk, processed, TrailingJunkAllowed);
+    auto r = qt_asciiToDouble(s00, len, TrailingJunkAllowed);
     if (se)
-        *se = s00 + processed;
+        *se = s00 + (r.used < 0 ? -r.used : r.used);
     if (ok)
-        *ok = nonNullOk;
-    return d;
+        *ok = r.ok();
+    return r.result;
 }
 
 QString qdtoa(qreal d, int *decpt, int *sign)
@@ -635,7 +601,7 @@ QString qdtoa(qreal d, int *decpt, int *sign)
     if (decpt)
         *decpt = nonNullDecpt;
 
-    return QLatin1String(result, length);
+    return QLatin1StringView(result, length);
 }
 
 static QLocaleData::DoubleForm resolveFormat(int precision, int decpt, qsizetype length)
@@ -700,7 +666,7 @@ static T dtoString(double d, QLocaleData::DoubleForm form, int precision, bool u
     int bufSize = 1;
     if (precision == QLocale::FloatingPointShortest)
         bufSize += D::max_digits10;
-    else if (form == QLocaleData::DFDecimal && qIsFinite(d))
+    else if (form == QLocaleData::DFDecimal && qt_is_finite(d))
         bufSize += wholePartSpace(qAbs(d)) + precision;
     else // Add extra digit due to different interpretations of precision.
         bufSize += qMax(2, precision) + 1; // Must also be big enough for "nan" or "inf"
@@ -711,11 +677,11 @@ static T dtoString(double d, QLocaleData::DoubleForm form, int precision, bool u
     bool negative = false;
     int length = 0;
     int decpt = 0;
-    qt_doubleToAscii(d, form, precision, buffer.data(), buffer.length(), negative, length, decpt);
-    QLatin1String view(buffer.data(), buffer.data() + length);
+    qt_doubleToAscii(d, form, precision, buffer.data(), buffer.size(), negative, length, decpt);
+    QLatin1StringView view(buffer.data(), length);
     const bool succinct = form == QLocaleData::DFSignificantDigits;
     qsizetype total = (negative ? 1 : 0) + length;
-    if (qIsFinite(d)) {
+    if (qt_is_finite(d)) {
         if (succinct)
             form = resolveFormat(precision, decpt, view.size());
 
@@ -757,7 +723,7 @@ static T dtoString(double d, QLocaleData::DoubleForm form, int precision, bool u
 
     if (negative && !isZero(d)) // We don't return "-0"
         result.append(Char('-'));
-    if (!qIsFinite(d)) {
+    if (!qt_is_finite(d)) {
         result.append(view);
         if (uppercase)
             result = std::move(result).toUpper();
@@ -778,7 +744,7 @@ static T dtoString(double d, QLocaleData::DoubleForm form, int precision, bool u
             result.append(Char(uppercase ? 'E' : 'e'));
             result.append(Char(exponent < 0 ? '-' : '+'));
             exponent = std::abs(exponent);
-            Q_ASSUME(exponent <= D::max_exponent10 + D::max_digits10);
+            Q_ASSERT(exponent <= D::max_exponent10 + D::max_digits10);
             int exponentDigits = digits(exponent);
             // C's printf guarantees a two-digit exponent, and so do we:
             if (exponentDigits == 1)
@@ -844,4 +810,79 @@ QByteArray qdtoAscii(double d, QLocaleData::DoubleForm form, int precision, bool
     return dtoString<QByteArray>(d, form, precision, uppercase);
 }
 
+#if defined(QT_SUPPORTS_INT128) || defined(QT_USE_MSVC_INT128)
+static inline quint64 toUInt64(qinternaluint128 v)
+{
+#if defined(QT_USE_MSVC_INT128)
+    return quint64(v._Word[0]);
+#else
+    return quint64(v);
+#endif
+}
+
+QString quint128toBasicLatin(qinternaluint128 number, int base)
+{
+    // We divide our 128-bit number into parts that we can do text
+    // concatenation with. This list is the maximum power of the
+    // base that is less than 2^64.
+    static constexpr auto dividers = []() constexpr {
+        std::array<quint64, 35> bases {};
+        for (int base = 2; base <= 36; ++base) {
+            quint64 v = base;
+            while (v * base > v)
+                v *= base;
+            bases[base - 2] = v;
+        }
+        return bases;
+    }();
+    static constexpr auto digitCounts = []() constexpr {
+        std::array<quint8, 35> digits{};
+        for (int base = 2; base <= 36; ++base) {
+            quint64 v = base;
+            int i = 0;
+            for (i = 0; v * base > v; ++i)
+                v *= base;
+            digits[base - 2] = i;
+        }
+        return digits;
+    }();
+
+    QString result;
+
+    constexpr unsigned flags = QLocaleData::NoFlags;
+    const QLocaleData *dd = QLocaleData::c();
+
+    // special base cases:
+    constexpr int Width = -1;
+    if (base == 2 || base == 4 || base == 16) {
+        // 2^64 is a power of 2, 4 and 16
+        result = dd->unsLongLongToString(quint64(number), 64, base, Width, flags);
+        result.prepend(dd->unsLongLongToString(quint64(number >> 64), -1, base, Width, flags));
+    } else {
+        int digitCount = digitCounts[base - 2];
+        quint64 divider = dividers[base - 2];
+        quint64 lower = toUInt64(number % divider);
+        number /= divider;
+        while (number) {
+            result.prepend(dd->unsLongLongToString(lower, digitCount, base, Width, flags));
+            lower = toUInt64(number % divider);
+            number /= divider;
+        }
+        result.prepend(dd->unsLongLongToString(lower, -1, base, Width, flags));
+    }
+    return result;
+}
+
+QString qint128toBasicLatin(qinternalint128 number, int base)
+{
+    const bool negative = number < 0;
+    if (negative)
+        number *= -1;
+    QString result = quint128toBasicLatin(qinternaluint128(number), base);
+    if (negative)
+        result.prepend(u'-');
+    return result;
+}
+#endif // defined(QT_SUPPORTS_INT128) || defined(QT_USE_MSVC_INT128)
+
 QT_END_NAMESPACE
