@@ -30,6 +30,7 @@
 #include <emscripten/val.h>
 
 #include <QtCore/private/qstdweb_p.h>
+#include <QKeySequence>
 
 QT_BEGIN_NAMESPACE
 
@@ -105,41 +106,27 @@ QWasmWindow::QWasmWindow(QWindow *w, QWasmDeadKeySupport *deadKeySupport,
 
     m_flags = window()->flags();
 
-    const auto pointerCallback = std::function([this](emscripten::val event) {
-        if (processPointer(*PointerEvent::fromWeb(event)))
-            event.call<void>("preventDefault");
-    });
+    m_pointerEnterCallback = std::make_unique<qstdweb::EventCallback>(m_qtWindow, "pointerenter",
+        [this](emscripten::val event) { this->handlePointerEvent(event); });
+    m_pointerLeaveCallback = std::make_unique<qstdweb::EventCallback>(m_qtWindow, "pointerleave",
+        [this](emscripten::val event) { this->handlePointerEvent(event); });
+    m_wheelEventCallback = std::make_unique<qstdweb::EventCallback>( m_qtWindow, "wheel",
+        [this](emscripten::val event) { this->handleWheelEvent(event); });
 
-    m_pointerEnterCallback =
-            std::make_unique<qstdweb::EventCallback>(m_qtWindow, "pointerenter", pointerCallback);
-    m_pointerLeaveCallback =
-            std::make_unique<qstdweb::EventCallback>(m_qtWindow, "pointerleave", pointerCallback);
-
-    m_wheelEventCallback = std::make_unique<qstdweb::EventCallback>(
-            m_qtWindow, "wheel", [this](emscripten::val event) {
-                if (processWheel(*WheelEvent::fromWeb(event)))
-                    event.call<void>("preventDefault");
-            });
-
-    const auto keyCallback = std::function([this](emscripten::val event) {
-        if (processKey(*KeyEvent::fromWebWithDeadKeyTranslation(event, m_deadKeySupport)))
-            event.call<void>("preventDefault");
-        event.call<void>("stopPropagation");
-    });
-
-   emscripten::val keyFocusWindow;
-    if (QWasmInputContext *wasmContext =
-       qobject_cast<QWasmInputContext *>(QWasmIntegration::get()->inputContext())) {
-        // if there is an touchscreen input context,
-        // use that window for key input
-        keyFocusWindow = wasmContext->m_inputElement;
-    } else {
-        keyFocusWindow = m_qtWindow;
+    QWasmInputContext *wasmInput = QWasmIntegration::get()->wasmInputContext();
+    if (wasmInput) {
+        m_keyDownCallbackForInputContext =
+            std::make_unique<qstdweb::EventCallback>(wasmInput->m_inputElement, "keydown",
+            [this](emscripten::val event) { this->handleKeyForInputContextEvent(event); });
+        m_keyUpCallbackForInputContext =
+            std::make_unique<qstdweb::EventCallback>(wasmInput->m_inputElement, "keyup",
+            [this](emscripten::val event) { this->handleKeyForInputContextEvent(event); });
     }
 
-    m_keyDownCallback =
-            std::make_unique<qstdweb::EventCallback>(keyFocusWindow, "keydown", keyCallback);
-    m_keyUpCallback = std::make_unique<qstdweb::EventCallback>(keyFocusWindow, "keyup", keyCallback);
+    m_keyDownCallback = std::make_unique<qstdweb::EventCallback>(m_qtWindow, "keydown",
+        [this](emscripten::val event) { this->handleKeyEvent(event); });
+    m_keyUpCallback =std::make_unique<qstdweb::EventCallback>(m_qtWindow, "keyup",
+        [this](emscripten::val event) { this->handleKeyEvent(event); });
 
     setParent(parent());
 }
@@ -207,23 +194,9 @@ bool QWasmWindow::onNonClientEvent(const PointerEvent &event)
 
 void QWasmWindow::initialize()
 {
-    QRect rect = windowGeometry();
-
-    const auto windowFlags = window()->flags();
-    const bool shouldRestrictMinSize =
-            !windowFlags.testFlag(Qt::FramelessWindowHint) && !windowIsPopupType(windowFlags);
-    const bool isMinSizeUninitialized = window()->minimumSize() == QSize(0, 0);
-
-    if (shouldRestrictMinSize && isMinSizeUninitialized)
-        window()->setMinimumSize(QSize(minSizeForRegularWindows, minSizeForRegularWindows));
-
-
-    const QSize minimumSize = windowMinimumSize();
-    const QSize maximumSize = windowMaximumSize();
-    const QSize targetSize = !rect.isEmpty() ? rect.size() : minimumSize;
-
-    rect.setWidth(qBound(minimumSize.width(), targetSize.width(), maximumSize.width()));
-    rect.setHeight(qBound(minimumSize.height(), targetSize.height(), maximumSize.height()));
+    auto initialGeometry = QPlatformWindow::initialGeometry(window(),
+            windowGeometry(), defaultWindowSize, defaultWindowSize);
+    m_normalGeometry = initialGeometry;
 
     setWindowState(window()->windowStates());
     setWindowFlags(window()->flags());
@@ -232,7 +205,6 @@ void QWasmWindow::initialize()
 
     if (window()->isTopLevel())
         setWindowIcon(window()->icon());
-    m_normalGeometry = rect;
     QPlatformWindow::setGeometry(m_normalGeometry);
 
 #if QT_CONFIG(accessibility)
@@ -371,8 +343,6 @@ void QWasmWindow::raise()
 {
     bringToTop();
     invalidate();
-    if (QWasmIntegration::get()->inputContext())
-        m_canvas.call<void>("focus");
 }
 
 void QWasmWindow::lower()
@@ -502,6 +472,14 @@ void QWasmWindow::commitParent(QWasmWindowTreeNode *parent)
     m_commitedParent = parent;
 }
 
+void QWasmWindow::handleKeyEvent(const emscripten::val &event)
+{
+    qCDebug(qLcQpaWasmInputContext) << "processKey as KeyEvent";
+    if (processKey(*KeyEvent::fromWebWithDeadKeyTranslation(event, m_deadKeySupport)))
+        event.call<void>("preventDefault");
+    event.call<void>("stopPropagation");
+}
+
 bool QWasmWindow::processKey(const KeyEvent &event)
 {
     constexpr bool ProceedToNativeEvent = false;
@@ -522,6 +500,88 @@ bool QWasmWindow::processKey(const KeyEvent &event)
             : result;
 }
 
+void QWasmWindow::handleKeyForInputContextEvent(const emscripten::val &event)
+{
+    QWasmInputContext *wasmInput = QWasmIntegration::get()->wasmInputContext();
+    if (wasmInput) {
+        const auto keyString = QString::fromStdString(event["key"].as<std::string>());
+        qCDebug(qLcQpaWasmInputContext) << "Key callback" << keyString << keyString.size();
+
+        if (keyString == "Unidentified") {
+            // Android makes a bunch of KeyEvents as "Unidentified"
+            // They will be processed just in InputContext.
+            return;
+        } else if (event["ctrlKey"].as<bool>()
+                || event["altKey"].as<bool>()
+                || event["metaKey"].as<bool>()) {
+            if (processKeyForInputContext(*KeyEvent::fromWebWithDeadKeyTranslation(event, m_deadKeySupport)))
+                event.call<void>("preventDefault");
+            event.call<void>("stopImmediatePropagation");
+            return;
+        } else if (keyString.size() != 1) {
+            if (!wasmInput->preeditString().isEmpty()) {
+                if (keyString == "Process" || keyString == "Backspace") {
+                    // processed by InputContext
+                    // "Process" should be handled by InputContext but
+                    // QWasmInputContext's function is incomplete now
+                    // so, there will be some exceptions here.
+                    return;
+                } else if (keyString != "Shift"
+                         && keyString != "Meta"
+                         && keyString != "Alt"
+                         && keyString != "Control"
+                         && !keyString.startsWith("Arrow")) {
+                    wasmInput->commitPreeditAndClear();
+                }
+            }
+        } else if (wasmInput->inputMethodAccepted()) {
+            // processed in inputContext with skipping processKey
+            return;
+        }
+    }
+
+    qCDebug(qLcQpaWasmInputContext) << "processKey as KeyEvent";
+    if (processKeyForInputContext(*KeyEvent::fromWebWithDeadKeyTranslation(event, m_deadKeySupport)))
+        event.call<void>("preventDefault");
+    event.call<void>("stopImmediatePropagation");
+}
+
+bool QWasmWindow::processKeyForInputContext(const KeyEvent &event)
+{
+    qCDebug(qLcQpaWasmInputContext) << Q_FUNC_INFO;
+    Q_ASSERT(event.type == EventType::KeyDown || event.type == EventType::KeyUp);
+
+    QKeySequence keySeq(event.modifiers | event.key);
+
+    if (keySeq == QKeySequence::Paste) {
+        // Process it in pasteCallback and inputCallback
+        return false;
+    }
+
+    const auto result = QWindowSystemInterface::handleKeyEvent(
+            0, event.type == EventType::KeyDown ? QEvent::KeyPress : QEvent::KeyRelease, event.key,
+            event.modifiers, event.text);
+
+    // Copy/Cut callback required to copy qtClipboard to system clipboard
+    if (keySeq == QKeySequence::Copy || keySeq == QKeySequence::Cut)
+        return false;
+
+    return result;
+}
+
+void QWasmWindow::handlePointerEvent(const emscripten::val &event)
+{
+    // Ideally it should not be happened but
+    // it takes place sometime with some reason
+    // without compositionend.
+    QWasmInputContext *wasmInput = QWasmIntegration::get()->wasmInputContext();
+    if (wasmInput && !wasmInput->preeditString().isEmpty())
+        wasmInput->commitPreeditAndClear();
+
+    if (processPointer(*PointerEvent::fromWeb(event)))
+        event.call<void>("preventDefault");
+}
+
 bool QWasmWindow::processPointer(const PointerEvent &event)
 {
     if (event.pointerType != PointerType::Mouse && event.pointerType != PointerType::Pen)
@@ -545,6 +605,12 @@ bool QWasmWindow::processPointer(const PointerEvent &event)
     return false;
 }
 
+void QWasmWindow::handleWheelEvent(const emscripten::val &event)
+{
+    if (processWheel(*WheelEvent::fromWeb(event)))
+        event.call<void>("preventDefault");
+}
+
 bool QWasmWindow::processWheel(const WheelEvent &event)
 {
     // Web scroll deltas are inverted from Qt deltas - negate.
@@ -677,6 +743,10 @@ void QWasmWindow::setMask(const QRegion &region)
 
 void QWasmWindow::setParent(const QPlatformWindow *)
 {
+    // The window flags depend on whether we are a
+    // child window or not, so update them here.
+    setWindowFlags(window()->flags());
+
     commitParent(parentNode());
 }
 
