@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the plugins of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qwindowsbackingstore.h"
 #include "qwindowswindow.h"
@@ -43,7 +7,7 @@
 
 #include <QtGui/qwindow.h>
 #include <QtGui/qpainter.h>
-#include <QtFontDatabaseSupport/private/qwindowsnativeimage_p.h>
+#include <QtGui/private/qwindowsnativeimage_p.h>
 #include <private/qhighdpiscaling_p.h>
 #include <private/qimage_p.h>
 
@@ -91,9 +55,8 @@ void QWindowsBackingStore::flush(QWindow *window, const QRegion &region,
     if ((flags & Qt::FramelessWindowHint) && QWindowsWindow::setWindowLayered(rw->handle(), flags, hasAlpha, rw->opacity()) && hasAlpha) {
         // Windows with alpha: Use blend function to update.
         QRect r = QHighDpi::toNativePixels(window->frameGeometry(), window);
-        QPoint frameOffset(QHighDpi::toNativePixels(QPoint(window->frameMargins().left(), window->frameMargins().top()),
-                                                    static_cast<const QWindow *>(nullptr)));
-        QRect dirtyRect = br.translated(offset + frameOffset);
+        QMargins frameMargins = rw->frameMargins();
+        QRect dirtyRect = br.translated(offset + QPoint(frameMargins.left(), frameMargins.top()));
 
         SIZE size = {r.width(), r.height()};
         POINT ptDst = {r.x(), r.y()};
@@ -154,7 +117,7 @@ void QWindowsBackingStore::resize(const QSize &size, const QRegion &region)
         if (QImage::toPixelFormat(format).alphaUsage() == QPixelFormat::UsesAlpha)
             m_alphaNeedsFill = true;
         else // upgrade but here we know app painting does not rely on alpha hence no need to fill
-            format = qt_maybeAlphaVersionWithSameDepth(format);
+            format = qt_maybeDataCompatibleAlphaVersion(format);
 
         QWindowsNativeImage *oldwni = m_image.data();
         auto *newwni = new QWindowsNativeImage(size.width(), size.height(), format);
@@ -183,8 +146,8 @@ bool QWindowsBackingStore::scroll(const QRegion &area, int dx, int dy)
         return false;
 
     const QPoint offset(dx, dy);
-    for (const QRect &rect : area)
-        qt_scrollRectInImage(m_image->image(), rect, offset);
+    const QRect rect = area.boundingRect();
+    qt_scrollRectInImage(m_image->image(), rect, offset);
 
     return true;
 }
