@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtWidgets module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qinputdialog.h"
 
@@ -54,6 +18,8 @@
 #include "qevent.h"
 #include "qdialog_p.h"
 
+#include <QtCore/qpointer.h>
+
 QT_USE_NAMESPACE
 
 enum CandidateSignal {
@@ -71,10 +37,10 @@ static const char *candidateSignal(int which)
     case IntValueSelectedSignal:    return SIGNAL(intValueSelected(int));
     case DoubleValueSelectedSignal: return SIGNAL(doubleValueSelected(double));
 
-    case NumCandidateSignals:       ; // fall through
+    case NumCandidateSignals:
+        break;
     };
-    Q_UNREACHABLE();
-    return Q_NULLPTR;
+    Q_UNREACHABLE_RETURN(nullptr);
 }
 
 static const char *signalForMember(const char *member)
@@ -104,8 +70,10 @@ class QInputDialogSpinBox : public QSpinBox
 public:
     QInputDialogSpinBox(QWidget *parent)
         : QSpinBox(parent) {
-        connect(lineEdit(), SIGNAL(textChanged(QString)), this, SLOT(notifyTextChanged()));
-        connect(this, SIGNAL(editingFinished()), this, SLOT(notifyTextChanged()));
+        connect(lineEdit(), &QLineEdit::textChanged,
+                this, &QInputDialogSpinBox::notifyTextChanged);
+        connect(this, &QInputDialogSpinBox::editingFinished,
+                this, &QInputDialogSpinBox::notifyTextChanged);
     }
 
 signals:
@@ -115,18 +83,16 @@ private slots:
     void notifyTextChanged() { emit textChanged(hasAcceptableInput()); }
 
 private:
-    void keyPressEvent(QKeyEvent *event) Q_DECL_OVERRIDE {
+    void keyPressEvent(QKeyEvent *event) override {
         if ((event->key() == Qt::Key_Return || event->key() == Qt::Key_Enter) && !hasAcceptableInput()) {
-#ifndef QT_NO_PROPERTIES
             setProperty("value", property("value"));
-#endif
         } else {
             QSpinBox::keyPressEvent(event);
         }
         notifyTextChanged();
     }
 
-    void mousePressEvent(QMouseEvent *event) Q_DECL_OVERRIDE {
+    void mousePressEvent(QMouseEvent *event) override {
         QSpinBox::mousePressEvent(event);
         notifyTextChanged();
     }
@@ -137,10 +103,12 @@ class QInputDialogDoubleSpinBox : public QDoubleSpinBox
     Q_OBJECT
 
 public:
-    QInputDialogDoubleSpinBox(QWidget *parent = 0)
+    QInputDialogDoubleSpinBox(QWidget *parent = nullptr)
         : QDoubleSpinBox(parent) {
-        connect(lineEdit(), SIGNAL(textChanged(QString)), this, SLOT(notifyTextChanged()));
-        connect(this, SIGNAL(editingFinished()), this, SLOT(notifyTextChanged()));
+        connect(lineEdit(), &QLineEdit::textChanged,
+                this, &QInputDialogDoubleSpinBox::notifyTextChanged);
+        connect(this, &QInputDialogDoubleSpinBox::editingFinished,
+                this, &QInputDialogDoubleSpinBox::notifyTextChanged);
     }
 
 signals:
@@ -150,23 +118,33 @@ private slots:
     void notifyTextChanged() { emit textChanged(hasAcceptableInput()); }
 
 private:
-    void keyPressEvent(QKeyEvent *event) Q_DECL_OVERRIDE {
+    void keyPressEvent(QKeyEvent *event) override {
         if ((event->key() == Qt::Key_Return || event->key() == Qt::Key_Enter) && !hasAcceptableInput()) {
-#ifndef QT_NO_PROPERTIES
             setProperty("value", property("value"));
-#endif
         } else {
             QDoubleSpinBox::keyPressEvent(event);
         }
         notifyTextChanged();
     }
 
-    void mousePressEvent(QMouseEvent *event) Q_DECL_OVERRIDE {
+    void mousePressEvent(QMouseEvent *event) override {
         QDoubleSpinBox::mousePressEvent(event);
         notifyTextChanged();
     }
 };
 
+class QInputDialogListView : public QListView
+{
+public:
+    QInputDialogListView(QWidget *parent = nullptr) : QListView(parent) {}
+    QVariant inputMethodQuery(Qt::InputMethodQuery query) const override
+    {
+        if (query == Qt::ImEnabled)
+            return false;
+        return QListView::inputMethodQuery(query);
+    }
+};
+
 class QInputDialogPrivate : public QDialogPrivate
 {
     Q_DECLARE_PUBLIC(QInputDialog)
@@ -189,9 +167,9 @@ public:
     QString listViewText() const;
     void ensureLayout() const { const_cast<QInputDialogPrivate *>(this)->ensureLayout(); }
     bool useComboBoxOrListView() const { return comboBox && comboBox->count() > 0; }
-    void _q_textChanged(const QString &text);
-    void _q_plainTextEditTextChanged();
-    void _q_currentRowChanged(const QModelIndex &newIndex, const QModelIndex &oldIndex);
+    void textChanged(const QString &text);
+    void plainTextEditTextChanged();
+    void currentRowChanged(const QModelIndex &newIndex, const QModelIndex &oldIndex);
 
     mutable QLabel *label;
     mutable QDialogButtonBox *buttonBox;
@@ -200,7 +178,7 @@ public:
     mutable QSpinBox *intSpinBox;
     mutable QDoubleSpinBox *doubleSpinBox;
     mutable QComboBox *comboBox;
-    mutable QListView *listView;
+    mutable QInputDialogListView *listView;
     mutable QWidget *inputWidget;
     mutable QVBoxLayout *mainLayout;
     QInputDialog::InputDialogOptions opts;
@@ -210,8 +188,8 @@ public:
 };
 
 QInputDialogPrivate::QInputDialogPrivate()
-    : label(0), buttonBox(0), lineEdit(0), plainTextEdit(0), intSpinBox(0), doubleSpinBox(0),
-      comboBox(0), listView(0), inputWidget(0), mainLayout(0)
+    : label(nullptr), buttonBox(nullptr), lineEdit(nullptr), plainTextEdit(nullptr), intSpinBox(nullptr), doubleSpinBox(nullptr),
+      comboBox(nullptr), listView(nullptr), inputWidget(nullptr), mainLayout(nullptr)
 {
 }
 
@@ -235,11 +213,10 @@ void QInputDialogPrivate::ensureLayout()
     label->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed);
 
     buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel, Qt::Horizontal, q);
-    QObject::connect(buttonBox, SIGNAL(accepted()), q, SLOT(accept()));
-    QObject::connect(buttonBox, SIGNAL(rejected()), q, SLOT(reject()));
+    QObject::connect(buttonBox, &QDialogButtonBox::accepted, q, &QDialog::accept);
+    QObject::connect(buttonBox, &QDialogButtonBox::rejected, q, &QDialog::reject);
 
     mainLayout = new QVBoxLayout(q);
-    //we want to let the input dialog grow to available size on Symbian.
     mainLayout->setSizeConstraint(QLayout::SetMinAndMaxSize);
     mainLayout->addWidget(label);
     mainLayout->addWidget(inputWidget);
@@ -257,8 +234,8 @@ void QInputDialogPrivate::ensureLineEdit()
         qt_widget_private(lineEdit)->inheritsInputMethodHints = 1;
 #endif
         lineEdit->hide();
-        QObject::connect(lineEdit, SIGNAL(textChanged(QString)),
-                         q, SLOT(_q_textChanged(QString)));
+        QObjectPrivate::connect(lineEdit, &QLineEdit::textChanged,
+                                this, &QInputDialogPrivate::textChanged);
     }
 }
 
@@ -272,8 +249,8 @@ void QInputDialogPrivate::ensurePlainTextEdit()
         qt_widget_private(plainTextEdit)->inheritsInputMethodHints = 1;
 #endif
         plainTextEdit->hide();
-        QObject::connect(plainTextEdit, SIGNAL(textChanged()),
-                         q, SLOT(_q_plainTextEditTextChanged()));
+        QObjectPrivate::connect(plainTextEdit, &QPlainTextEdit::textChanged,
+                                this, &QInputDialogPrivate::plainTextEditTextChanged);
     }
 }
 
@@ -286,10 +263,10 @@ void QInputDialogPrivate::ensureComboBox()
         qt_widget_private(comboBox)->inheritsInputMethodHints = 1;
 #endif
         comboBox->hide();
-        QObject::connect(comboBox, SIGNAL(editTextChanged(QString)),
-                         q, SLOT(_q_textChanged(QString)));
-        QObject::connect(comboBox, SIGNAL(currentIndexChanged(QString)),
-                         q, SLOT(_q_textChanged(QString)));
+        QObjectPrivate::connect(comboBox, &QComboBox::editTextChanged,
+                                this, &QInputDialogPrivate::textChanged);
+        QObjectPrivate::connect(comboBox, &QComboBox::currentTextChanged,
+                                this, &QInputDialogPrivate::textChanged);
     }
 }
 
@@ -298,16 +275,15 @@ void QInputDialogPrivate::ensureListView()
     Q_Q(QInputDialog);
     if (!listView) {
         ensureComboBox();
-
-        listView = new QListView(q);
+        listView = new QInputDialogListView(q);
         listView->hide();
         listView->setEditTriggers(QAbstractItemView::NoEditTriggers);
         listView->setSelectionMode(QAbstractItemView::SingleSelection);
         listView->setModel(comboBox->model());
         listView->setCurrentIndex(QModelIndex()); // ###
-        QObject::connect(listView->selectionModel(),
-                         SIGNAL(currentRowChanged(QModelIndex,QModelIndex)),
-                         q, SLOT(_q_currentRowChanged(QModelIndex,QModelIndex)));
+        QObjectPrivate::connect(listView->selectionModel(),
+                                &QItemSelectionModel::currentRowChanged,
+                                this, &QInputDialogPrivate::currentRowChanged);
     }
 }
 
@@ -317,8 +293,8 @@ void QInputDialogPrivate::ensureIntSpinBox()
     if (!intSpinBox) {
         intSpinBox = new QInputDialogSpinBox(q);
         intSpinBox->hide();
-        QObject::connect(intSpinBox, SIGNAL(valueChanged(int)),
-                         q, SIGNAL(intValueChanged(int)));
+        QObject::connect(intSpinBox, &QInputDialogSpinBox::valueChanged,
+                         q, &QInputDialog::intValueChanged);
     }
 }
 
@@ -328,8 +304,8 @@ void QInputDialogPrivate::ensureDoubleSpinBox()
     if (!doubleSpinBox) {
         doubleSpinBox = new QInputDialogDoubleSpinBox(q);
         doubleSpinBox->hide();
-        QObject::connect(doubleSpinBox, SIGNAL(valueChanged(double)),
-                         q, SIGNAL(doubleValueChanged(double)));
+        QObject::connect(doubleSpinBox, &QInputDialogDoubleSpinBox::valueChanged,
+                         q, &QInputDialog::doubleValueChanged);
     }
 }
 
@@ -404,9 +380,9 @@ void QInputDialogPrivate::chooseRightTextInputWidget()
     setInputWidget(widget);
 
     if (inputWidget == comboBox) {
-        _q_textChanged(comboBox->currentText());
+        textChanged(comboBox->currentText());
     } else if (inputWidget == listView) {
-        _q_textChanged(listViewText());
+        textChanged(listViewText());
     }
 }
 
@@ -440,7 +416,7 @@ QString QInputDialogPrivate::listViewText() const
     }
 }
 
-void QInputDialogPrivate::_q_textChanged(const QString &text)
+void QInputDialogPrivate::textChanged(const QString &text)
 {
     Q_Q(QInputDialog);
     if (textValue != text) {
@@ -449,7 +425,7 @@ void QInputDialogPrivate::_q_textChanged(const QString &text)
     }
 }
 
-void QInputDialogPrivate::_q_plainTextEditTextChanged()
+void QInputDialogPrivate::plainTextEditTextChanged()
 {
     Q_Q(QInputDialog);
     QString text = plainTextEdit->toPlainText();
@@ -459,10 +435,10 @@ void QInputDialogPrivate::_q_plainTextEditTextChanged()
     }
 }
 
-void QInputDialogPrivate::_q_currentRowChanged(const QModelIndex &newIndex,
-                                               const QModelIndex & /* oldIndex */)
+void QInputDialogPrivate::currentRowChanged(const QModelIndex &newIndex,
+                                            const QModelIndex & /* oldIndex */)
 {
-    _q_textChanged(comboBox->model()->data(newIndex).toString());
+    textChanged(comboBox->model()->data(newIndex).toString());
     buttonBox->button(QDialogButtonBox::Ok)->setEnabled(true);
 }
 
@@ -495,7 +471,6 @@ void QInputDialogPrivate::_q_currentRowChanged(const QModelIndex &newIndex,
 
 /*!
     \enum QInputDialog::InputMode
-    \since 4.5
 
     This enum describes the different modes of input that can be selected for
     the dialog.
@@ -509,8 +484,6 @@ void QInputDialogPrivate::_q_currentRowChanged(const QModelIndex &newIndex,
 */
 
 /*!
-    \since 4.5
-
     Constructs a new input dialog with the given \a parent and window \a flags.
 */
 QInputDialog::QInputDialog(QWidget *parent, Qt::WindowFlags flags)
@@ -519,8 +492,6 @@ QInputDialog::QInputDialog(QWidget *parent, Qt::WindowFlags flags)
 }
 
 /*!
-    \since 4.5
-
     Destroys the input dialog.
 */
 QInputDialog::~QInputDialog()
@@ -528,8 +499,6 @@ QInputDialog::~QInputDialog()
 }
 
 /*!
-    \since 4.5
-
     \property QInputDialog::inputMode
 
     \brief the mode used for input
@@ -583,8 +552,6 @@ QInputDialog::InputMode QInputDialog::inputMode() const
 }
 
 /*!
-    \since 4.5
-
     \property QInputDialog::labelText
 
     \brief the label's text which describes what needs to be input
@@ -609,8 +576,6 @@ QString QInputDialog::labelText() const
 /*!
     \enum QInputDialog::InputDialogOption
 
-    \since 4.5
-
     This enum specifies various options that affect the look and feel
     of an input dialog.
 
@@ -651,7 +616,6 @@ bool QInputDialog::testOption(InputDialogOption option) const
 /*!
     \property QInputDialog::options
     \brief the various options that affect the look and feel of the dialog
-    \since 4.5
 
     By default, all options are disabled.
 
@@ -683,8 +647,6 @@ QInputDialog::InputDialogOptions QInputDialog::options() const
 }
 
 /*!
-    \since 4.5
-
     \property QInputDialog::textValue
 
     \brief the text value for the input dialog
@@ -715,8 +677,6 @@ QString QInputDialog::textValue() const
 }
 
 /*!
-    \since 4.5
-
     \property QInputDialog::textEchoMode
 
     \brief the echo mode for the text value
@@ -742,8 +702,6 @@ QLineEdit::EchoMode QInputDialog::textEchoMode() const
 }
 
 /*!
-    \since 4.5
-
     \property QInputDialog::comboBoxEditable
 
     \brief whether or not the combo box used in the input dialog is editable
@@ -768,8 +726,6 @@ bool QInputDialog::isComboBoxEditable() const
 }
 
 /*!
-    \since 4.5
-
     \property QInputDialog::comboBoxItems
 
     \brief the items used in the combo box for the input dialog
@@ -804,7 +760,6 @@ QStringList QInputDialog::comboBoxItems() const
 
 /*!
     \property QInputDialog::intValue
-    \since 4.5
     \brief the current integer value accepted as input
 
     This property is only relevant when the input dialog is used in
@@ -829,7 +784,6 @@ int QInputDialog::intValue() const
 
 /*!
     \property QInputDialog::intMinimum
-    \since 4.5
     \brief the minimum integer value accepted as input
 
     This property is only relevant when the input dialog is used in
@@ -854,7 +808,6 @@ int QInputDialog::intMinimum() const
 
 /*!
     \property QInputDialog::intMaximum
-    \since 4.5
     \brief the maximum integer value accepted as input
 
     This property is only relevant when the input dialog is used in
@@ -891,7 +844,6 @@ void QInputDialog::setIntRange(int min, int max)
 
 /*!
     \property QInputDialog::intStep
-    \since 4.5
     \brief the step by which the integer value is increased and decreased
 
     This property is only relevant when the input dialog is used in
@@ -916,7 +868,6 @@ int QInputDialog::intStep() const
 
 /*!
     \property QInputDialog::doubleValue
-    \since 4.5
     \brief the current double precision floating point value accepted as input
 
     This property is only relevant when the input dialog is used in
@@ -941,7 +892,6 @@ double QInputDialog::doubleValue() const
 
 /*!
     \property QInputDialog::doubleMinimum
-    \since 4.5
     \brief the minimum double precision floating point value accepted as input
 
     This property is only relevant when the input dialog is used in
@@ -966,7 +916,6 @@ double QInputDialog::doubleMinimum() const
 
 /*!
     \property QInputDialog::doubleMaximum
-    \since 4.5
     \brief the maximum double precision floating point value accepted as input
 
     This property is only relevant when the input dialog is used in
@@ -1002,8 +951,6 @@ void QInputDialog::setDoubleRange(double min, double max)
 }
 
 /*!
-    \since 4.5
-
     \property QInputDialog::doubleDecimals
 
     \brief sets the precision of the double spinbox in decimals
@@ -1028,8 +975,6 @@ int QInputDialog::doubleDecimals() const
 }
 
 /*!
-    \since 4.5
-
     \property QInputDialog::okButtonText
 
     \brief the text for the button used to accept the entry in the dialog
@@ -1049,8 +994,6 @@ QString QInputDialog::okButtonText() const
 }
 
 /*!
-    \since 4.5
-
     \property QInputDialog::cancelButtonText
     \brief the text for the button used to cancel the dialog
 */
@@ -1069,9 +1012,6 @@ QString QInputDialog::cancelButtonText() const
 }
 
 /*!
-    \since 4.5
-    \overload
-
     This function connects one of its signals to the slot specified by \a receiver
     and \a member. The specific signal depends on the arguments that are specified
     in \a member. These are:
@@ -1164,7 +1104,7 @@ void QInputDialog::done(int result)
     if (d->receiverToDisconnectOnClose) {
         disconnect(this, signalForMember(d->memberToDisconnectOnClose),
                    d->receiverToDisconnectOnClose, d->memberToDisconnectOnClose);
-        d->receiverToDisconnectOnClose = 0;
+        d->receiverToDisconnectOnClose = nullptr;
     }
     d->memberToDisconnectOnClose.clear();
 }
@@ -1180,7 +1120,7 @@ void QInputDialog::done(int result)
     \a inputMethodHints is the input method hints that will be used in the
     edit widget if an input method is active.
 
-    If \a ok is nonnull \e *\a ok will be set to true if the user pressed
+    If \a ok is nonnull \e {*ok} will be set to true if the user pressed
     \uicontrol OK and to false if the user pressed \uicontrol Cancel. The dialog's parent
     is \a parent. The dialog will be modal and uses the specified widget
     \a flags.
@@ -1228,7 +1168,7 @@ QString QInputDialog::getText(QWidget *parent, const QString &title, const QStri
     \a inputMethodHints is the input method hints that will be used in the
     edit widget if an input method is active.
 
-    If \a ok is nonnull \e *\a ok will be set to true if the user pressed
+    If \a ok is nonnull \e {*ok} will be set to true if the user pressed
     \uicontrol OK and to false if the user pressed \uicontrol Cancel. The dialog's parent
     is \a parent. The dialog will be modal and uses the specified widget
     \a flags.
@@ -1265,8 +1205,6 @@ QString QInputDialog::getMultiLineText(QWidget *parent, const QString &title, co
 }
 
 /*!
-    \since 4.5
-
     Static convenience function to get an integer input from the user.
 
     \a title is the text which is displayed in the title bar of the dialog.
@@ -1312,67 +1250,6 @@ int QInputDialog::getInt(QWidget *parent, const QString &title, const QString &l
 }
 
 /*!
-    \fn QInputDialog::getInteger(QWidget *parent, const QString &title, const QString &label, int value, int min, int max, int step, bool *ok, Qt::WindowFlags flags)
-    \deprecated use getInt()
-
-    Static convenience function to get an integer input from the user.
-
-    \a title is the text which is displayed in the title bar of the dialog.
-    \a label is the text which is shown to the user (it should say what should
-    be entered).
-    \a value is the default integer which the spinbox will be set to.
-    \a min and \a max are the minimum and maximum values the user may choose.
-    \a step is the amount by which the values change as the user presses the
-    arrow buttons to increment or decrement the value.
-
-    If \a ok is nonnull *\a ok will be set to true if the user pressed \uicontrol OK
-    and to false if the user pressed \uicontrol Cancel. The dialog's parent is
-    \a parent. The dialog will be modal and uses the widget \a flags.
-
-    On success, this function returns the integer which has been entered by the
-    user; on failure, it returns the initial \a value.
-
-    Use this static function like this:
-
-    \snippet dialogs/standarddialogs/dialog.cpp 0
-
-    \sa getText(), getDouble(), getItem(), getMultiLineText()
-*/
-
-/*!
-    Static convenience function to get a floating point number from the user.
-
-    \a title is the text which is displayed in the title bar of the dialog.
-    \a label is the text which is shown to the user (it should say what should
-    be entered).
-    \a value is the default floating point number that the line edit will be
-    set to.
-    \a min and \a max are the minimum and maximum values the user may choose.
-    \a decimals is the maximum number of decimal places the number may have.
-
-    If \a ok is nonnull, *\a ok will be set to true if the user pressed \uicontrol OK
-    and to false if the user pressed \uicontrol Cancel. The dialog's parent is
-    \a parent. The dialog will be modal and uses the widget \a flags.
-
-    This function returns the floating point number which has been entered by
-    the user.
-
-    Use this static function like this:
-
-    \snippet dialogs/standarddialogs/dialog.cpp 1
-
-    \sa getText(), getInt(), getItem(), getMultiLineText()
-*/
-
-double QInputDialog::getDouble(QWidget *parent, const QString &title, const QString &label,
-                               double value, double min, double max, int decimals, bool *ok,
-                               Qt::WindowFlags flags)
-{
-    return QInputDialog::getDouble(parent, title, label, value, min, max, decimals, ok, flags, 1.0);
-}
-
-/*!
-    \overload
     Static convenience function to get a floating point number from the user.
 
     \a title is the text which is displayed in the title bar of the dialog.
@@ -1436,7 +1313,7 @@ double QInputDialog::getDouble(QWidget *parent, const QString &title, const QStr
     If \a editable is true the user can enter their own text; otherwise, the
     user may only select one of the existing items.
 
-    If \a ok is nonnull \e *\a ok will be set to true if the user pressed
+    If \a ok is nonnull \e {*ok} will be set to true if the user pressed
     \uicontrol OK and to false if the user pressed \uicontrol Cancel. The dialog's parent
     is \a parent. The dialog will be modal and uses the widget \a flags.
 
