@@ -1,8 +1,10 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 
 #include <QTest>
+#include <QtTest/private/qcomparisontesthelper_p.h>
+
 #if QT_CONFIG(process)
 #include <QProcess>
 #endif
@@ -17,6 +19,7 @@ class tst_QUuid : public QObject
 private slots:
     void initTestCase();
 
+    void compareCompiles();
     void fromChar();
     void toString();
     void fromString_data();
@@ -33,12 +36,16 @@ private slots:
     void equal();
     void notEqual();
     void cpp11();
+    void ordering_data();
+    void ordering();
 
     // Only in Qt > 3.2.x
     void generate();
     void less();
     void more();
+    void variants_data();
     void variants();
+    void versions_data();
     void versions();
 
     void threadUniqueness();
@@ -89,20 +96,28 @@ void tst_QUuid::initTestCase()
     uuidD = QUuid(0x21f7f8de, 0x8051, 0x5b89, 0x86, 0x80, 0x01, 0x95, 0xef, 0x79, 0x8b, 0x6a);
 }
 
+void tst_QUuid::compareCompiles()
+{
+    QTestPrivate::testAllComparisonOperatorsCompile<QUuid>();
+#if defined(Q_OS_WIN)
+    QTestPrivate::testEqualityOperatorsCompile<QUuid, GUID>();
+#endif
+}
+
 void tst_QUuid::fromChar()
 {
-    QCOMPARE(uuidA, QUuid("{fc69b59e-cc34-4436-a43c-ee95d128b8c5}"));
-    QCOMPARE(uuidA, QUuid("fc69b59e-cc34-4436-a43c-ee95d128b8c5}"));
-    QCOMPARE(uuidA, QUuid("{fc69b59e-cc34-4436-a43c-ee95d128b8c5"));
-    QCOMPARE(uuidA, QUuid("fc69b59e-cc34-4436-a43c-ee95d128b8c5"));
-    QCOMPARE(QUuid(), QUuid("{fc69b59e-cc34-4436-a43c-ee95d128b8c"));
-    QCOMPARE(QUuid(), QUuid("{fc69b59e-cc34"));
-    QCOMPARE(QUuid(), QUuid("fc69b59e-cc34-"));
-    QCOMPARE(QUuid(), QUuid("fc69b59e-cc34"));
-    QCOMPARE(QUuid(), QUuid("cc34"));
-    QCOMPARE(QUuid(), QUuid(nullptr));
+    QT_TEST_EQUALITY_OPS(uuidA, QUuid("{fc69b59e-cc34-4436-a43c-ee95d128b8c5}"), true);
+    QT_TEST_EQUALITY_OPS(uuidA, QUuid("fc69b59e-cc34-4436-a43c-ee95d128b8c5}"), true);
+    QT_TEST_EQUALITY_OPS(uuidA, QUuid("{fc69b59e-cc34-4436-a43c-ee95d128b8c5"), true);
+    QT_TEST_EQUALITY_OPS(uuidA, QUuid("fc69b59e-cc34-4436-a43c-ee95d128b8c5"), true);
+    QT_TEST_EQUALITY_OPS(QUuid(), QUuid("{fc69b59e-cc34-4436-a43c-ee95d128b8c"), true);
+    QT_TEST_EQUALITY_OPS(QUuid(), QUuid("{fc69b59e-cc34"), true);
+    QT_TEST_EQUALITY_OPS(QUuid(), QUuid("fc69b59e-cc34-"), true);
+    QT_TEST_EQUALITY_OPS(QUuid(), QUuid("fc69b59e-cc34"), true);
+    QT_TEST_EQUALITY_OPS(QUuid(), QUuid("cc34"), true);
+    QT_TEST_EQUALITY_OPS(QUuid(), QUuid(nullptr), true);
 
-    QCOMPARE(uuidB, QUuid(QString("{1ab6e93a-b1cb-4a87-ba47-ec7e99039a7b}")));
+    QT_TEST_EQUALITY_OPS(uuidB, QUuid(QString("{1ab6e93a-b1cb-4a87-ba47-ec7e99039a7b}")), true);
 }
 
 void tst_QUuid::toString()
@@ -162,21 +177,21 @@ void tst_QUuid::fromString()
     const auto inputL1 = input.toLatin1();
     const auto inputU8 = input.toUtf8();
 
-    QCOMPARE(expected, QUuid(input));
-    QCOMPARE(expected, QUuid(inputU8));
-    QCOMPARE(expected, QUuid(inputL1));
+    QT_TEST_EQUALITY_OPS(expected, QUuid(input), true);
+    QT_TEST_EQUALITY_OPS(expected, QUuid(inputU8), true);
+    QT_TEST_EQUALITY_OPS(expected, QUuid(inputL1), true);
 
-    QCOMPARE(expected, QUuid::fromString(input));
+    QT_TEST_EQUALITY_OPS(expected, QUuid::fromString(input), true);
 
     // for QLatin1String, construct one whose data() is not NUL-terminated:
     const auto longerInputL1 = inputL1 + '5'; // the '5' makes the premature end check incorrectly succeed
     const auto inputL1S = QLatin1String(longerInputL1.data(), inputL1.size());
-    QCOMPARE(expected, QUuid::fromString(inputL1S));
+    QT_TEST_EQUALITY_OPS(expected, QUuid::fromString(inputL1S), true);
 
     // for QUtf8StringView, too:
     const auto longerInputU8 = inputU8 + '5'; // the '5' makes the premature end check incorrectly succeed
     const auto inputU8S = QUtf8StringView(longerInputU8.data(), inputU8.size());
-    QCOMPARE(expected, QUuid::fromString(inputU8S));
+    QT_TEST_EQUALITY_OPS(expected, QUuid::fromString(inputU8S), true);
 }
 
 void tst_QUuid::toByteArray()
@@ -196,27 +211,30 @@ void tst_QUuid::toByteArray()
 
 void tst_QUuid::fromByteArray()
 {
-    QCOMPARE(uuidA, QUuid(QByteArray("{fc69b59e-cc34-4436-a43c-ee95d128b8c5}")));
-    QCOMPARE(uuidA, QUuid(QByteArray("fc69b59e-cc34-4436-a43c-ee95d128b8c5}")));
-    QCOMPARE(uuidA, QUuid(QByteArray("{fc69b59e-cc34-4436-a43c-ee95d128b8c5")));
-    QCOMPARE(uuidA, QUuid(QByteArray("fc69b59e-cc34-4436-a43c-ee95d128b8c5")));
-    QCOMPARE(QUuid(), QUuid(QByteArray("{fc69b59e-cc34-4436-a43c-ee95d128b8c")));
+    QT_TEST_EQUALITY_OPS(uuidA, QUuid(QByteArray("{fc69b59e-cc34-4436-a43c-ee95d128b8c5}")), true);
+    QT_TEST_EQUALITY_OPS(uuidA, QUuid(QByteArray("fc69b59e-cc34-4436-a43c-ee95d128b8c5}")), true);
+    QT_TEST_EQUALITY_OPS(uuidA, QUuid(QByteArray("{fc69b59e-cc34-4436-a43c-ee95d128b8c5")), true);
+    QT_TEST_EQUALITY_OPS(uuidA, QUuid(QByteArray("fc69b59e-cc34-4436-a43c-ee95d128b8c5")), true);
+    QT_TEST_EQUALITY_OPS(QUuid(), QUuid(QByteArray("{fc69b59e-cc34-4436-a43c-ee95d128b8c")), true);
 
-    QCOMPARE(uuidB, QUuid(QByteArray("{1ab6e93a-b1cb-4a87-ba47-ec7e99039a7b}")));
+    QT_TEST_EQUALITY_OPS(uuidB, QUuid(QByteArray("{1ab6e93a-b1cb-4a87-ba47-ec7e99039a7b}")), true);
 }
 
 void tst_QUuid::toRfc4122()
 {
     QCOMPARE(uuidA.toRfc4122(), QByteArray::fromHex("fc69b59ecc344436a43cee95d128b8c5"));
-
     QCOMPARE(uuidB.toRfc4122(), QByteArray::fromHex("1ab6e93ab1cb4a87ba47ec7e99039a7b"));
 }
 
 void tst_QUuid::fromRfc4122()
 {
-    QCOMPARE(uuidA, QUuid::fromRfc4122(QByteArray::fromHex("fc69b59ecc344436a43cee95d128b8c5")));
+    QT_TEST_EQUALITY_OPS(
+            uuidA,
+            QUuid::fromRfc4122(QByteArray::fromHex("fc69b59ecc344436a43cee95d128b8c5")), true);
 
-    QCOMPARE(uuidB, QUuid::fromRfc4122(QByteArray::fromHex("1ab6e93ab1cb4a87ba47ec7e99039a7b")));
+    QT_TEST_EQUALITY_OPS(
+            uuidB, QUuid::fromRfc4122(QByteArray::fromHex("1ab6e93ab1cb4a87ba47ec7e99039a7b")),
+            true);
 }
 
 void tst_QUuid::id128()
@@ -234,15 +252,15 @@ void tst_QUuid::id128()
         0xba, 0x47, 0xec, 0x7e, 0x99, 0x03, 0x9a, 0x7b,
     } };
 
-    QCOMPARE(QUuid(bytesA), uuidA);
-    QCOMPARE(QUuid(bytesB), uuidB);
+    QT_TEST_EQUALITY_OPS(QUuid(bytesA), uuidA, true);
+    QT_TEST_EQUALITY_OPS(QUuid(bytesB), uuidB, true);
     QVERIFY(memcmp(uuidA.toBytes().data, bytesA.data, sizeof(QUuid::Id128Bytes)) == 0);
     QVERIFY(memcmp(uuidB.toBytes().data, bytesB.data, sizeof(QUuid::Id128Bytes)) == 0);
 
     QUuid::Id128Bytes leBytesA = {};
     for (int i = 0; i < 16; i++)
         leBytesA.data[15 - i] = bytesA.data[i];
-    QCOMPARE(QUuid(leBytesA, QSysInfo::LittleEndian), uuidA);
+    QT_TEST_EQUALITY_OPS(QUuid(leBytesA, QSysInfo::LittleEndian), uuidA, true);
     QVERIFY(memcmp(uuidA.toBytes(QSysInfo::LittleEndian).data, leBytesA.data, sizeof(leBytesA)) == 0);
 
     // check the new q{To,From}{Big,Little}Endian() overloads
@@ -266,24 +284,21 @@ void tst_QUuid::id128()
 void tst_QUuid::uint128()
 {
 #ifdef QT_SUPPORTS_INT128
-    constexpr quint128 u = quint128(Q_UINT64_C(0xfc69b59ecc344436)) << 64
-                            | Q_UINT64_C(0xa43cee95d128b8c5); // This is LE
+    constexpr quint128 u = Q_UINT128_C(0xfc69b59e'cc344436'a43cee95'd128b8c5); // This is LE
     constexpr quint128 be = qToBigEndian(u);
     constexpr QUuid uuid = QUuid::fromUInt128(be);
     static_assert(uuid.toUInt128() == be, "Round-trip through QUuid failed");
 
-    QCOMPARE(uuid, uuidA);
-    QCOMPARE(quint64(uuid.toUInt128() >> 64), quint64(be >> 64));
-    QCOMPARE(quint64(uuid.toUInt128()), quint64(be));
+    QT_TEST_EQUALITY_OPS(uuid, uuidA, true);
+    QCOMPARE(uuid.toUInt128(), be);
 
     quint128 le = qFromBigEndian(be);
-    QCOMPARE(quint64(uuid.toUInt128(QSysInfo::LittleEndian) >> 64), quint64(le >> 64));
-    QCOMPARE(quint64(uuid.toUInt128(QSysInfo::LittleEndian)), quint64(le));
-    QCOMPARE(QUuid::fromUInt128(le, QSysInfo::LittleEndian), uuidA);
+    QCOMPARE(uuid.toUInt128(QSysInfo::LittleEndian), le);
+    QT_TEST_EQUALITY_OPS(QUuid::fromUInt128(le, QSysInfo::LittleEndian), uuidA, true);
 
     QUuid::Id128Bytes bytes = { .data128 = { qToBigEndian(u) } };
     QUuid uuid2(bytes);
-    QCOMPARE(uuid2, uuid);
+    QT_TEST_EQUALITY_OPS(uuid2, uuid, true);
 
     // verify that toBytes() and toUInt128() provide bytewise similar result
     constexpr quint128 val = uuid.toUInt128();
@@ -297,11 +312,11 @@ void tst_QUuid::uint128()
 void tst_QUuid::createUuidV3OrV5()
 {
     //"www.widgets.com" is also from RFC4122
-    QCOMPARE(uuidC, QUuid::createUuidV3(uuidNS, QByteArray("www.widgets.com")));
-    QCOMPARE(uuidC, QUuid::createUuidV3(uuidNS, QString("www.widgets.com")));
+    QT_TEST_EQUALITY_OPS(uuidC, QUuid::createUuidV3(uuidNS, QByteArray("www.widgets.com")), true);
+    QT_TEST_EQUALITY_OPS(uuidC, QUuid::createUuidV3(uuidNS, QString("www.widgets.com")), true);
 
-    QCOMPARE(uuidD, QUuid::createUuidV5(uuidNS, QByteArray("www.widgets.com")));
-    QCOMPARE(uuidD, QUuid::createUuidV5(uuidNS, QString("www.widgets.com")));
+    QT_TEST_EQUALITY_OPS(uuidD, QUuid::createUuidV5(uuidNS, QByteArray("www.widgets.com")), true);
+    QT_TEST_EQUALITY_OPS(uuidD, QUuid::createUuidV5(uuidNS, QString("www.widgets.com")), true);
 }
 
 void tst_QUuid::check_QDataStream()
@@ -317,7 +332,7 @@ void tst_QUuid::check_QDataStream()
         QDataStream in(&ar,QIODevice::ReadOnly);
         in.setByteOrder(QDataStream::BigEndian);
         in >> tmp;
-        QCOMPARE(uuidA, tmp);
+        QT_TEST_EQUALITY_OPS(uuidA, tmp, true);
     }
     {
         QDataStream out(&ar,QIODevice::WriteOnly);
@@ -328,7 +343,7 @@ void tst_QUuid::check_QDataStream()
         QDataStream in(&ar,QIODevice::ReadOnly);
         in.setByteOrder(QDataStream::LittleEndian);
         in >> tmp;
-        QCOMPARE(uuidA, tmp);
+        QT_TEST_EQUALITY_OPS(uuidA, tmp, true);
     }
 }
 
@@ -343,14 +358,14 @@ void tst_QUuid::isNull()
 
 void tst_QUuid::equal()
 {
-    QVERIFY( !(uuidA == uuidB) );
+    QT_TEST_EQUALITY_OPS(uuidA, uuidB, false);
 
     QUuid copy(uuidA);
-    QCOMPARE(uuidA, copy);
+    QT_TEST_EQUALITY_OPS(uuidA, copy, true);
 
     QUuid assigned;
     assigned = uuidA;
-    QCOMPARE(uuidA, assigned);
+    QT_TEST_EQUALITY_OPS(uuidA, assigned, true);
 }
 
 
@@ -371,6 +386,123 @@ void tst_QUuid::cpp11() {
 #endif
 }
 
+constexpr QUuid make_minimal(QUuid::Variant variant)
+{
+    using V = QUuid::Variant;
+    switch (variant) {
+    case V::VarUnknown: // special case
+        return {};
+    case V::NCS:        // special case: null would be NCS, but is treated as Unknown
+        return {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1};
+    case V::DCE:        // special case: DCE should be 0b100, but is 0b10
+        return {0, 0, 0, 0b1000'0000, 0, 0, 0, 0, 0, 0, 0};
+    case V::Microsoft:
+    case V::Reserved:
+        return {0, 0, 0, uchar(variant << 5), 0, 0, 0, 0, 0, 0, 0};
+    }
+}
+
+void tst_QUuid::ordering_data()
+{
+    QTest::addColumn<QUuid>("lhs");
+    QTest::addColumn<QUuid>("rhs");
+    QTest::addColumn<Qt::strong_ordering>("expected");
+
+    // QUuid is sorted by variant() first, then the dataN fields, in order
+    // Exhaustive testing is pointless, so pick some strategic values
+
+    constexpr QUuid null = make_minimal(QUuid::Variant::VarUnknown);
+    QCOMPARE(null.variant(), QUuid::Variant::VarUnknown);
+
+    constexpr QUuid minNCS = make_minimal(QUuid::Variant::NCS);
+    QCOMPARE(minNCS.variant(), QUuid::Variant::NCS);
+
+    constexpr QUuid ncs000_0000_0001 = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1};
+    QCOMPARE(ncs000_0000_0001, minNCS);
+    constexpr QUuid ncs000_0000_0010 = {0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0};
+    constexpr QUuid ncs000_0000_0100 = {0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0};
+    constexpr QUuid ncs000_0000_1000 = {0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0};
+
+    constexpr QUuid ncs000_0001_0000 = {0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0};
+    constexpr QUuid ncs000_0010_0000 = {0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0};
+    constexpr QUuid ncs000_0100_0000 = {0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0};
+    constexpr QUuid ncs000_1000_0000 = {0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0};
+
+    constexpr QUuid ncs001_0000_0000 = {0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0};
+    constexpr QUuid ncs010_0000_0000 = {0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0};
+    constexpr QUuid ncs100_0000_0000 = {1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
+
+    constexpr QUuid minDCE = make_minimal(QUuid::Variant::DCE);
+    QCOMPARE(minDCE.variant(), QUuid::Variant::DCE);
+
+    constexpr QUuid minMS = make_minimal(QUuid::Variant::Microsoft);
+    QCOMPARE(minMS.variant(), QUuid::Variant::Microsoft);
+
+    constexpr QUuid minR = make_minimal(QUuid::Variant::Reserved);
+    QCOMPARE(minR.variant(), QUuid::Variant::Reserved);
+
+    constexpr QUuid ones = {0xFFFF'FFFFU, 0xFFFFu, 0xFFFFu, 0xFFu, 0xFFu, 0xFFu, 0xFFu, 0xFFu, 0xFFu, 0xFFu, 0xFFu};
+    QCOMPARE(ones.variant(), QUuid::Variant::Reserved);
+
+#define ROW(l, r, c) \
+    QTest::addRow("%s<>%s", #l, #r) << l << r << Qt::strong_ordering:: c \
+    /* end */
+#define EQUAL(x) ROW(x, x, equal)
+    EQUAL(null);
+    EQUAL(minNCS);
+    EQUAL(minDCE);
+    EQUAL(minMS);
+    EQUAL(minR);
+    EQUAL(ones);
+#undef EQUAL
+#define AFTER_NULL(x) ROW(null, x, less)
+    AFTER_NULL(minNCS);
+    AFTER_NULL(minDCE);
+    AFTER_NULL(minMS);
+    AFTER_NULL(minR);
+    AFTER_NULL(ones);
+#undef AFTER_NULL
+#define AFTER_NCS(x) ROW(minNCS, x, less)
+    AFTER_NCS(ncs000_0000_0010);
+    AFTER_NCS(ncs000_0000_0100);
+    AFTER_NCS(ncs000_0000_1000);
+    AFTER_NCS(ncs000_0001_0000);
+    AFTER_NCS(ncs000_0010_0000);
+    AFTER_NCS(ncs000_0100_0000);
+    AFTER_NCS(ncs000_1000_0000);
+    AFTER_NCS(ncs001_0000_0000);
+    AFTER_NCS(ncs010_0000_0000);
+    AFTER_NCS(ncs100_0000_0000);
+    ROW(ncs100_0000_0000, minDCE, less);
+    AFTER_NCS(minDCE);
+    AFTER_NCS(minMS);
+    AFTER_NCS(minR);
+    AFTER_NCS(ones);
+#undef AFTER_NCS
+#define AFTER_DCE(x) ROW(minDCE, x, less)
+    AFTER_DCE(minMS);
+    AFTER_DCE(minR);
+    AFTER_DCE(ones);
+#undef AFTER_DCE
+#define AFTER_MS(x) ROW(minMS, x, less)
+    AFTER_MS(minR);
+    AFTER_MS(ones);
+#undef AFTER_MS
+#define AFTER_R(x) ROW(minR, x, less)
+    AFTER_R(ones);
+#undef AFTER_R
+#undef ROW
+}
+
+void tst_QUuid::ordering()
+{
+    QFETCH(const QUuid, lhs);
+    QFETCH(const QUuid, rhs);
+    QFETCH(const Qt::strong_ordering, expected);
+
+    QT_TEST_ALL_COMPARISON_OPS(lhs, rhs, expected);
+}
+
 void tst_QUuid::generate()
 {
     QUuid shouldnt_be_null_uuidA = QUuid::createUuid();
@@ -387,10 +519,12 @@ void tst_QUuid::less()
     QVERIFY(  uuidB <= uuidA);
     QVERIFY(!(uuidA <  uuidB) );
     QVERIFY(!(uuidA <= uuidB));
+    QT_TEST_ALL_COMPARISON_OPS(uuidB, uuidA, Qt::strong_ordering::less);
 
     QUuid null_uuid;
     QVERIFY(null_uuid < uuidA); // Null uuid is always less than a valid one
     QVERIFY(null_uuid <= uuidA);
+    QT_TEST_ALL_COMPARISON_OPS(null_uuid, uuidA, Qt::strong_ordering::less);
 
     QVERIFY(null_uuid <= null_uuid);
     QVERIFY(uuidA <= uuidA);
@@ -403,6 +537,7 @@ void tst_QUuid::more()
     QVERIFY(  uuidA >= uuidB);
     QVERIFY(!(uuidB >  uuidA));
     QVERIFY(!(uuidB >= uuidA));
+    QT_TEST_ALL_COMPARISON_OPS(uuidA, uuidB, Qt::strong_ordering::greater);
 
     QUuid null_uuid;
     QVERIFY(!(null_uuid >  uuidA)); // Null uuid is always less than a valid one
@@ -410,29 +545,76 @@ void tst_QUuid::more()
 
     QVERIFY(null_uuid >= null_uuid);
     QVERIFY(uuidA >= uuidA);
+    QT_TEST_ALL_COMPARISON_OPS(uuidA, uuidA, Qt::strong_ordering::equal);
 }
 
+void tst_QUuid::variants_data()
+{
+    QTest::addColumn<QUuid>("uuid");
+    QTest::addColumn<QUuid::Variant>("variant");
+
+    QTest::newRow("default-constructed") << QUuid() << QUuid::VarUnknown;
+    QTest::newRow("minimal-NCS") << make_minimal(QUuid::NCS) << QUuid::NCS;
+    QTest::newRow("minimal-DCE") << make_minimal(QUuid::DCE) << QUuid::DCE;
+    QTest::newRow("minimal-Microsoft") << make_minimal(QUuid::Microsoft) << QUuid::Microsoft;
+    QTest::newRow("minimal-Reserved") << make_minimal(QUuid::Reserved) << QUuid::Reserved;
+    QTest::newRow("uuidA") << uuidA << QUuid::DCE;
+    QTest::newRow("uuidB") << uuidB << QUuid::DCE;
+    QTest::newRow("NCS") << QUuid("{3a2f883c-4000-000d-0000-00fb40000000}") << QUuid::NCS;
+}
 
 void tst_QUuid::variants()
 {
-    QVERIFY( uuidA.variant() == QUuid::DCE );
-    QVERIFY( uuidB.variant() == QUuid::DCE );
+    QFETCH(const QUuid, uuid);
+    QFETCH(const QUuid::Variant, variant);
 
-    QUuid NCS("{3a2f883c-4000-000d-0000-00fb40000000}");
-    QVERIFY( NCS.variant() == QUuid::NCS );
+    QCOMPARE_EQ(uuid.variant(), variant);
 }
 
+void tst_QUuid::versions_data()
+{
+    QTest::addColumn<QUuid>("uuid");
+    QTest::addColumn<QUuid::Version>("version");
+
+    QTest::newRow("default-constructed") << QUuid() << QUuid::VerUnknown;
+    QTest::newRow("DCE-time") << QUuid("{406c45a0-3b7e-11d0-80a3-0000c08810a7}") << QUuid::Time;
+    QTest::newRow("DCE-EmbPosix")
+            << QUuid(0, 0, 0b0010'0010'1000'0010, 0b1010'0000, 0, 0, 0, 0, 0, 0, 0)
+            << QUuid::EmbeddedPOSIX;
+    QTest::newRow("DCE-Md5")
+            << QUuid(0, 0, 0b0011'0001'0100'1001, 0b1011'0000, 0, 0, 0, 0, 0, 0, 0)
+            << QUuid::Md5;
+    QTest::newRow("DCE-Random")
+            << QUuid(0, 0, 0b0100'0101'0001'1101, 0b1000'0000, 0, 0, 0, 0, 0, 0, 0)
+            << QUuid::Random;
+    QTest::newRow("DCE-Sha1")
+            << QUuid(0, 0, 0b0101'1101'0101'1011, 0b1001'0000, 0, 0, 0, 0, 0, 0, 0)
+            << QUuid::Sha1;
+    QTest::newRow("DCE-inv-less-than-Time->unknown")
+            << QUuid(0, 0, 0b0000'1101'0101'1011, 0b1000'0000, 0, 0, 0, 0, 0, 0, 0)
+            << QUuid::VerUnknown;
+    QTest::newRow("DCE-inv-greater-than-Sha1->unknown")
+            << QUuid(0, 0, 0b0111'1101'0101'1011, 0b1000'0000, 0, 0, 0, 0, 0, 0, 0)
+            << QUuid::VerUnknown;
+    QTest::newRow("NCS-Time->unknown")
+            << QUuid(0, 0, 0b0001'0000'0000'0000, 0b0100'0000, 0, 0, 0, 0, 0, 0, 0)
+            << QUuid::VerUnknown;
+    QTest::newRow("MS-Sha1->unknown")
+            << QUuid(0, 0, 0b0101'0000'0000'0000, 0b1100'0000, 0, 0, 0, 0, 0, 0, 0)
+            << QUuid::VerUnknown;
+    QTest::newRow("Reserved-Random->unknown")
+            << QUuid(0, 0, 0b0100'0000'0000'0000, 0b1110'0000, 0, 0, 0, 0, 0, 0, 0)
+            << QUuid::VerUnknown;
+    QTest::newRow("uuidA") << uuidA << QUuid::Random;
+    QTest::newRow("uuidB") << uuidB << QUuid::Random;
+}
 
 void tst_QUuid::versions()
 {
-    QVERIFY( uuidA.version() == QUuid::Random );
-    QVERIFY( uuidB.version() == QUuid::Random );
+    QFETCH(const QUuid, uuid);
+    QFETCH(const QUuid::Version, version);
 
-    QUuid DCE_time("{406c45a0-3b7e-11d0-80a3-0000c08810a7}");
-    QVERIFY( DCE_time.version() == QUuid::Time );
-
-    QUuid NCS("{3a2f883c-4000-000d-0000-00fb40000000}");
-    QVERIFY( NCS.version() == QUuid::VerUnknown );
+    QCOMPARE_EQ(uuid.version(), version);
 }
 
 class UuidThread : public QThread
@@ -515,7 +697,7 @@ void tst_QUuid::qvariant()
 
     QUuid uuid2 = v.value<QUuid>();
     QVERIFY(!uuid2.isNull());
-    QCOMPARE(uuid, uuid2);
+    QT_TEST_EQUALITY_OPS(uuid, uuid2, true);
 }
 
 void tst_QUuid::qvariant_conversion()
@@ -547,7 +729,7 @@ void tst_QUuid::qvariant_conversion()
         QVariant sv = QVariant::fromValue(uuid.toByteArray());
         QCOMPARE(sv.metaType(), QMetaType(QMetaType::QByteArray));
         QVERIFY(sv.canConvert<QUuid>());
-        QCOMPARE(sv.value<QUuid>(), uuid);
+        QT_TEST_EQUALITY_OPS(sv.value<QUuid>(), uuid, true);
     }
 }
 
